"""
Python module generated from Java source file org.bukkit.inventory.meta.SpawnEggMeta

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.entity import EntitySnapshot
from org.bukkit.entity import EntityType
from org.bukkit.inventory.meta import *
from typing import Any, Callable, Iterable, Tuple


class SpawnEggMeta(ItemMeta):
    """
    Represents a spawn egg and it's spawned type.
    """

    def getSpawnedType(self) -> "EntityType":
        """
        Get the type of entity this egg will spawn.

        Returns
        - The entity type. May be null for implementation specific default.

        Deprecated
        - different types are different items
        """
        ...


    def setSpawnedType(self, type: "EntityType") -> None:
        """
        Set the type of entity this egg will spawn.

        Arguments
        - type: The entity type. May be null for implementation specific
        default.

        Deprecated
        - different types are different items
        """
        ...


    def getSpawnedEntity(self) -> "EntitySnapshot":
        """
        Gets the EntitySnapshot that will be spawned by this spawn egg or null if no entity
        has been set. 
        
        All applicable data from the egg will be copied, such as custom name, health,
        and velocity. 

        Returns
        - the entity snapshot or null if no entity has been set
        """
        ...


    def setSpawnedEntity(self, snapshot: "EntitySnapshot") -> None:
        """
        Sets the EntitySnapshot that will be spawned by this spawn egg. 
        
        All applicable data from the entity will be copied, such as custom name,
        health, and velocity. 

        Arguments
        - snapshot: the snapshot
        """
        ...


    def clone(self) -> "SpawnEggMeta":
        ...



"""
Python module generated from Java source file org.bukkit.inventory.meta.trim.TrimMaterial

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import Keyed
from org.bukkit import Material
from org.bukkit import NamespacedKey
from org.bukkit import Registry
from org.bukkit import Translatable
from org.bukkit.inventory.meta.trim import *
from org.bukkit.registry import RegistryAware
from typing import Any, Callable, Iterable, Tuple


class TrimMaterial(Keyed, Translatable, RegistryAware):
    """
    Represents a material that may be used in an ArmorTrim.
    """

    QUARTZ = getTrimMaterial("quartz")
    """
    Material.QUARTZ.
    """
    IRON = getTrimMaterial("iron")
    """
    Material.IRON_INGOT.
    """
    NETHERITE = getTrimMaterial("netherite")
    """
    Material.NETHERITE_INGOT.
    """
    REDSTONE = getTrimMaterial("redstone")
    """
    Material.REDSTONE.
    """
    COPPER = getTrimMaterial("copper")
    """
    Material.COPPER_INGOT.
    """
    GOLD = getTrimMaterial("gold")
    """
    Material.GOLD_INGOT.
    """
    EMERALD = getTrimMaterial("emerald")
    """
    Material.EMERALD.
    """
    DIAMOND = getTrimMaterial("diamond")
    """
    Material.DIAMOND.
    """
    LAPIS = getTrimMaterial("lapis")
    """
    Material.LAPIS_LAZULI.
    """
    AMETHYST = getTrimMaterial("amethyst")
    """
    Material.AMETHYST_SHARD.
    """
    RESIN = getTrimMaterial("resin")
    """
    Material.RESIN_BRICK.
    """


    @staticmethod
    def getTrimMaterial(key: str) -> "TrimMaterial":
        ...


    def getKey(self) -> "NamespacedKey":
        """
        See
        - .isRegistered()

        Deprecated
        - A key might not always be present, use .getKeyOrThrow() instead.
        """
        ...



"""
Python module generated from Java source file org.bukkit.BanEntry

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util import Date
from org.bukkit import *
from typing import Any, Callable, Iterable, Tuple


class BanEntry:
    """
    A single entry from a ban list. This may represent either a player ban or
    an IP ban.
    
    Ban entries include the following properties:
    <table border=1>
    <caption>Property information</caption>
    <tr>
        <th>Property</th>
        <th>Description</th>
    </tr><tr>
        <td>Target Profile / IP Address</td>
        <td>The target profile or IP address</td>
    </tr><tr>
        <td>Creation Date</td>
        <td>The creation date of the ban</td>
    </tr><tr>
        <td>Source</td>
        <td>The source of the ban, such as a player, console, plugin, etc</td>
    </tr><tr>
        <td>Expiration Date</td>
        <td>The expiration date of the ban</td>
    </tr><tr>
        <td>Reason</td>
        <td>The reason for the ban</td>
    </tr>
    </table>
    
    Unsaved information is not automatically written to the implementation's
    ban list, instead, the .save() method must be called to write the
    changes to the ban list. If this ban entry has expired (such as from an
    unban) and is no longer found in the list, the .save() call will
    re-add it to the list, therefore banning the victim specified.
    
    Likewise, changes to the associated BanList or other entries may or
    may not be reflected in this entry.
    
    Type `<T>`: The ban target
    """

    def getTarget(self) -> str:
        """
        Gets the target involved. This may be in the form of an IP or a player
        name.

        Returns
        - the target name or IP address

        Deprecated
        - See .getBanTarget()
        """
        ...


    def getBanTarget(self) -> "T":
        """
        Gets the target involved.

        Returns
        - the target profile or IP address
        """
        ...


    def getCreated(self) -> "Date":
        """
        Gets the date this ban entry was created.

        Returns
        - the creation date
        """
        ...


    def setCreated(self, created: "Date") -> None:
        """
        Sets the date this ban entry was created.

        Arguments
        - created: the new created date, cannot be null

        See
        - .save() saving changes
        """
        ...


    def getSource(self) -> str:
        """
        Gets the source of this ban.
        
        Note: A source is considered any String, although this is generally a
        player name.

        Returns
        - the source of the ban
        """
        ...


    def setSource(self, source: str) -> None:
        """
        Sets the source of this ban.
        
        Note: A source is considered any String, although this is generally a
        player name.

        Arguments
        - source: the new source where null values become empty strings

        See
        - .save() saving changes
        """
        ...


    def getExpiration(self) -> "Date":
        """
        Gets the date this ban expires on, or null for no defined end date.

        Returns
        - the expiration date
        """
        ...


    def setExpiration(self, expiration: "Date") -> None:
        """
        Sets the date this ban expires on. Null values are considered
        "infinite" bans.

        Arguments
        - expiration: the new expiration date, or null to indicate an
            eternity

        See
        - .save() saving changes
        """
        ...


    def getReason(self) -> str:
        """
        Gets the reason for this ban.

        Returns
        - the ban reason, or null if not set
        """
        ...


    def setReason(self, reason: str) -> None:
        """
        Sets the reason for this ban. Reasons must not be null.

        Arguments
        - reason: the new reason, null values assume the implementation
            default

        See
        - .save() saving changes
        """
        ...


    def save(self) -> None:
        """
        Saves the ban entry, overwriting any previous data in the ban list.
        
        Saving the ban entry of an unbanned player will cause the player to be
        banned once again.
        """
        ...


    def remove(self) -> None:
        """
        Removes this ban entry from the appropriate ban list.
        """
        ...



"""
Python module generated from Java source file org.bukkit.material.Bed

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import Material
from org.bukkit.block import BlockFace
from org.bukkit.material import *
from typing import Any, Callable, Iterable, Tuple


class Bed(MaterialData, Directional):
    """
    Represents a bed.

    Deprecated
    - all usage of MaterialData is deprecated and subject to removal.
    Use org.bukkit.block.data.BlockData.
    """

    def __init__(self):
        """
        Default constructor for a bed.
        """
        ...


    def __init__(self, direction: "BlockFace"):
        """
        Instantiate a bed facing in a particular direction.

        Arguments
        - direction: the direction the bed's head is facing
        """
        ...


    def __init__(self, type: "Material"):
        ...


    def __init__(self, type: "Material", data: int):
        """
        Arguments
        - type: the type
        - data: the raw data value

        Deprecated
        - Magic value
        """
        ...


    def isHeadOfBed(self) -> bool:
        """
        Determine if this block represents the head of the bed

        Returns
        - True if this is the head of the bed, False if it is the foot
        """
        ...


    def setHeadOfBed(self, isHeadOfBed: bool) -> None:
        """
        Configure this to be either the head or the foot of the bed

        Arguments
        - isHeadOfBed: True to make it the head.
        """
        ...


    def setFacingDirection(self, face: "BlockFace") -> None:
        """
        Set which direction the head of the bed is facing. Note that this will
        only affect one of the two blocks the bed is made of.
        """
        ...


    def getFacing(self) -> "BlockFace":
        """
        Get the direction that this bed's head is facing toward

        Returns
        - the direction the head of the bed is facing
        """
        ...


    def toString(self) -> str:
        ...


    def clone(self) -> "Bed":
        ...



"""
Python module generated from Java source file com.google.gson.internal.UnsafeAllocator

Java source file obtained from artifact gson version 2.11.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.gson.internal import *
from java.io import ObjectInputStream
from java.io import ObjectStreamClass
from java.lang.reflect import Field
from java.lang.reflect import Method
from typing import Any, Callable, Iterable, Tuple


class UnsafeAllocator:
    """
    Do sneaky things to allocate objects without invoking their constructors.

    Author(s)
    - Jesse Wilson
    """

    INSTANCE = create()


    def newInstance(self, c: type["T"]) -> "T":
        ...



"""
Python module generated from Java source file java.lang.reflect.Proxy

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.lang.invoke import MethodHandle
from java.lang.invoke import MethodHandles
from java.lang.invoke import MethodType
from java.lang.invoke import WrongMethodTypeException
from java.lang.module import ModuleDescriptor
from java.lang.reflect import *
from java.security import AccessController
from java.security import PrivilegedAction
from java.util import ArrayDeque
from java.util import Arrays
from java.util import Collections
from java.util import Deque
from java.util import IdentityHashMap
from java.util import Objects
from java.util.concurrent import ConcurrentHashMap
from java.util.concurrent.atomic import AtomicInteger
from java.util.concurrent.atomic import AtomicLong
from java.util.function import BooleanSupplier
from jdk.internal.access import JavaLangAccess
from jdk.internal.access import SharedSecrets
from jdk.internal.loader import ClassLoaderValue
from jdk.internal.misc import VM
from jdk.internal.module import Modules
from jdk.internal.reflect import CallerSensitive
from jdk.internal.reflect import Reflection
from jdk.internal.vm.annotation import Stable
from sun.reflect.misc import ReflectUtil
from sun.security.action import GetPropertyAction
from sun.security.util import SecurityConstants
from typing import Any, Callable, Iterable, Tuple


class Proxy(Serializable):
    """
    `Proxy` provides static methods for creating objects that act like instances
    of interfaces but allow for customized method invocation.
    To create a proxy instance for some interface `Foo`:
    ````InvocationHandler handler = new MyInvocationHandler(...);
        Foo f = (Foo) Proxy.newProxyInstance(Foo.class.getClassLoader(),
                                             new Class<?>[] { Foo.class`,
                                             handler);
    }```
    
    
    A *proxy class* is a class created at runtime that implements a specified
    list of interfaces, known as *proxy interfaces*. A *proxy instance*
    is an instance of a proxy class.
    
    Each proxy instance has an associated *invocation handler*
    object, which implements the interface InvocationHandler.
    A method invocation on a proxy instance through one of its proxy
    interfaces will be dispatched to the InvocationHandler.invoke
    invoke method of the instance's invocation handler, passing the proxy
    instance, a `java.lang.reflect.Method` object identifying
    the method that was invoked, and an array of type `Object`
    containing the arguments.  The invocation handler processes the
    encoded method invocation as appropriate and the result that it
    returns will be returned as the result of the method invocation on
    the proxy instance.
    
    A proxy class has the following properties:
    
    
    - The unqualified name of a proxy class is unspecified.  The space
    of class names that begin with the string `"$Proxy"`
    should be, however, reserved for proxy classes.
    
    - The package and module in which a proxy class is defined is specified
    <a href="#membership">below</a>.
    
    - A proxy class is *final and non-abstract*.
    
    - A proxy class extends `java.lang.reflect.Proxy`.
    
    - A proxy class implements exactly the interfaces specified at its
    creation, in the same order. Invoking Class.getInterfaces() getInterfaces
    on its `Class` object will return an array containing the same
    list of interfaces (in the order specified at its creation), invoking
    Class.getMethods getMethods on its `Class` object will return
    an array of `Method` objects that include all of the
    methods in those interfaces, and invoking `getMethod` will
    find methods in the proxy interfaces as would be expected.
    
    - The java.security.ProtectionDomain of a proxy class
    is the same as that of system classes loaded by the bootstrap class
    loader, such as `java.lang.Object`, because the code for a
    proxy class is generated by trusted system code.  This protection
    domain will typically be granted `java.security.AllPermission`.
    
    - The Proxy.isProxyClass Proxy.isProxyClass method can be used
    to determine if a given class is a proxy class.
    
    
    A proxy instance has the following properties:
    
    
    - Given a proxy instance `proxy` and one of the
    interfaces, `Foo`, implemented by its proxy class, the
    following expression will return True:
    ```
        `proxy instanceof Foo`
    ```
    and the following cast operation will succeed (rather than throwing
    a `ClassCastException`):
    ```
        `(Foo) proxy`
    ```
    
    - Each proxy instance has an associated invocation handler, the one
    that was passed to its constructor.  The static
    Proxy.getInvocationHandler Proxy.getInvocationHandler method
    will return the invocation handler associated with the proxy instance
    passed as its argument.
    
    - An interface method invocation on a proxy instance will be
    encoded and dispatched to the invocation handler's InvocationHandler.invoke invoke method as described in the
    documentation for that method.
    
    - A proxy interface may define a default method or inherit
    a default method from its superinterface directly or indirectly.
    An invocation handler can invoke a default method of a proxy interface
    by calling InvocationHandler.invokeDefault(Object, Method, Object...)
    InvocationHandler::invokeDefault.
    
    - An invocation of the `hashCode`,
    `equals`, or `toString` methods declared in
    `java.lang.Object` on a proxy instance will be encoded and
    dispatched to the invocation handler's `invoke` method in
    the same manner as interface method invocations are encoded and
    dispatched, as described above.  The declaring class of the
    `Method` object passed to `invoke` will be
    `java.lang.Object`.  Other public methods of a proxy
    instance inherited from `java.lang.Object` are not
    overridden by a proxy class, so invocations of those methods behave
    like they do for instances of `java.lang.Object`.
    
    
    <h2><a id="membership">Package and Module Membership of Proxy Class</a></h2>
    
    The package and module to which a proxy class belongs are chosen such that
    the accessibility of the proxy class is in line with the accessibility of
    the proxy interfaces. Specifically, the package and the module membership
    of a proxy class defined via the
    Proxy.getProxyClass(ClassLoader, Class[]) or
    Proxy.newProxyInstance(ClassLoader, Class[], InvocationHandler)
    methods is specified as follows:
    
    <ol>
    - If all the proxy interfaces are in *exported* or *open*
        packages:
    <ol type="a">
    - if all the proxy interfaces are *public*, then the proxy class is
        *public* in an unconditionally exported but non-open package.
        The name of the package and the module are unspecified.
    
    - if at least one of all the proxy interfaces is *non-public*, then
        the proxy class is *non-public* in the package and module of the
        non-public interfaces. All the non-public interfaces must be in the same
        package and module; otherwise, proxying them is
        <a href="#restrictions">not possible</a>.
    </ol>
    
    - If at least one proxy interface is in a package that is
        *non-exported* and *non-open*:
    <ol type="a">
    - if all the proxy interfaces are *public*, then the proxy class is
        *public* in a *non-exported*, *non-open* package of
        <a href="#dynamicmodule">*dynamic module*.</a>
        The names of the package and the module are unspecified.
    
    - if at least one of all the proxy interfaces is *non-public*, then
        the proxy class is *non-public* in the package and module of the
        non-public interfaces. All the non-public interfaces must be in the same
        package and module; otherwise, proxying them is
        <a href="#restrictions">not possible</a>.
    </ol>
    
    </ol>
    
    
    Note that if proxy interfaces with a mix of accessibilities -- for example,
    an exported public interface and a non-exported non-public interface -- are
    proxied by the same instance, then the proxy class's accessibility is
    governed by the least accessible proxy interface.
    
    Note that it is possible for arbitrary code to obtain access to a proxy class
    in an open package with AccessibleObject.setAccessible setAccessible,
    whereas a proxy class in a non-open package is never accessible to
    code outside the module of the proxy class.
    
    
    Throughout this specification, a "non-exported package" refers to a package
    that is not exported to all modules, and a "non-open package" refers to
    a package that is not open to all modules.  Specifically, these terms refer to
    a package that either is not exported/open by its containing module or is
    exported/open in a qualified fashion by its containing module.
    
    <h3><a id="dynamicmodule">Dynamic Modules</a></h3>
    
    A dynamic module is a named module generated at runtime. A proxy class
    defined in a dynamic module is encapsulated and not accessible to any module.
    Calling Constructor.newInstance(Object...) on a proxy class in
    a dynamic module will throw `IllegalAccessException`;
    `Proxy.newProxyInstance` method should be used instead.
    
    
    A dynamic module can read the modules of all of the superinterfaces of a proxy
    class and the modules of the classes and interfaces referenced by
    all public method signatures of a proxy class.  If a superinterface or
    a referenced class or interface, say `T`, is in a non-exported package,
    the Module module of `T` is updated to export the
    package of `T` to the dynamic module.
    
    <h3>Methods Duplicated in Multiple Proxy Interfaces</h3>
    
    When two or more proxy interfaces contain a method with
    the same name and parameter signature, the order of the proxy class's
    interfaces becomes significant.  When such a *duplicate method*
    is invoked on a proxy instance, the `Method` object passed
    to the invocation handler will not necessarily be the one whose
    declaring class is assignable from the reference type of the interface
    that the proxy's method was invoked through.  This limitation exists
    because the corresponding method implementation in the generated proxy
    class cannot determine which interface it was invoked through.
    Therefore, when a duplicate method is invoked on a proxy instance,
    the `Method` object for the method in the foremost interface
    that contains the method (either directly or inherited through a
    superinterface) in the proxy class's list of interfaces is passed to
    the invocation handler's `invoke` method, regardless of the
    reference type through which the method invocation occurred.
    
    If a proxy interface contains a method with the same name and
    parameter signature as the `hashCode`, `equals`,
    or `toString` methods of `java.lang.Object`,
    when such a method is invoked on a proxy instance, the
    `Method` object passed to the invocation handler will have
    `java.lang.Object` as its declaring class.  In other words,
    the public, non-final methods of `java.lang.Object`
    logically precede all of the proxy interfaces for the determination of
    which `Method` object to pass to the invocation handler.
    
    Note also that when a duplicate method is dispatched to an
    invocation handler, the `invoke` method may only throw
    checked exception types that are assignable to one of the exception
    types in the `throws` clause of the method in *all* of
    the proxy interfaces that it can be invoked through.  If the
    `invoke` method throws a checked exception that is not
    assignable to any of the exception types declared by the method in one
    of the proxy interfaces that it can be invoked through, then an
    unchecked `UndeclaredThrowableException` will be thrown by
    the invocation on the proxy instance.  This restriction means that not
    all of the exception types returned by invoking
    `getExceptionTypes` on the `Method` object
    passed to the `invoke` method can necessarily be thrown
    successfully by the `invoke` method.

    Author(s)
    - Peter Jones

    See
    - InvocationHandler

    Since
    - 1.3

    Unknown Tags
    - 9
    """

    @staticmethod
    def getProxyClass(loader: "ClassLoader", *interfaces: Tuple[type[Any], ...]) -> type[Any]:
        """
        Returns the `java.lang.Class` object for a proxy class
        given a class loader and an array of interfaces.  The proxy class
        will be defined by the specified class loader and will implement
        all of the supplied interfaces.  If any of the given interfaces
        is non-public, the proxy class will be non-public. If a proxy class
        for the same permutation of interfaces has already been defined by the
        class loader, then the existing proxy class will be returned; otherwise,
        a proxy class for those interfaces will be generated dynamically
        and defined by the class loader.

        Arguments
        - loader: the class loader to define the proxy class
        - interfaces: the list of interfaces for the proxy class
                 to implement

        Returns
        - a proxy class that is defined in the specified class loader
                 and that implements the specified interfaces

        Raises
        - IllegalArgumentException: if any of the <a href="#restrictions">
                 restrictions</a> on the parameters are violated
        - SecurityException: if a security manager, *s*, is present
                 and any of the following conditions is met:
                 
                    -  the given `loader` is `null` and
                    the caller's class loader is not `null` and the
                    invocation of SecurityManager.checkPermission
                    s.checkPermission with
                    `RuntimePermission("getClassLoader")` permission
                    denies access.
                    -  for each proxy interface, `intf`,
                    the caller's class loader is not the same as or an
                    ancestor of the class loader for `intf` and
                    invocation of SecurityManager.checkPackageAccess
                    s.checkPackageAccess() denies access to `intf`.
                 
        - NullPointerException: if the `interfaces` array
                 argument or any of its elements are `null`

        See
        - <a href=".membership">Package and Module Membership of Proxy Class</a>

        Deprecated
        - Proxy classes generated in a named module are encapsulated
             and not accessible to code outside its module.
             Constructor.newInstance(Object...) Constructor.newInstance
             will throw `IllegalAccessException` when it is called on
             an inaccessible proxy class.
             Use .newProxyInstance(ClassLoader, Class[], InvocationHandler)
             to create a proxy instance instead.

        Unknown Tags
        - 9
        """
        ...


    @staticmethod
    def newProxyInstance(loader: "ClassLoader", interfaces: list[type[Any]], h: "InvocationHandler") -> "Object":
        """
        Returns a proxy instance for the specified interfaces
        that dispatches method invocations to the specified invocation
        handler.
        
        <a id="restrictions">`IllegalArgumentException` will be thrown
        if any of the following restrictions is violated:</a>
        
        - All of `Class` objects in the given `interfaces` array
        must represent Class.isHidden() non-hidden and
        Class.isSealed() non-sealed interfaces,
        not classes or primitive types.
        
        - No two elements in the `interfaces` array may
        refer to identical `Class` objects.
        
        - All of the interface types must be visible by name through the
        specified class loader. In other words, for class loader
        `cl` and every interface `i`, the following
        expression must be True:
        `Class.forName(i.getName(), False, cl) == i`
        
        - All of the types referenced by all
        public method signatures of the specified interfaces
        and those inherited by their superinterfaces
        must be visible by name through the specified class loader.
        
        - All non-public interfaces must be in the same package
        and module, defined by the specified class loader and
        the module of the non-public interfaces can access all of
        the interface types; otherwise, it would not be possible for
        the proxy class to implement all of the interfaces,
        regardless of what package it is defined in.
        
        - For any set of member methods of the specified interfaces
        that have the same signature:
        
        - If the return type of any of the methods is a primitive
        type or void, then all of the methods must have that same
        return type.
        - Otherwise, one of the methods must have a return type that
        is assignable to all of the return types of the rest of the
        methods.
        
        
        - The resulting proxy class must not exceed any limits imposed
        on classes by the virtual machine.  For example, the VM may limit
        the number of interfaces that a class may implement to 65535; in
        that case, the size of the `interfaces` array must not
        exceed 65535.
        
        
        Note that the order of the specified proxy interfaces is
        significant: two requests for a proxy class with the same combination
        of interfaces but in a different order will result in two distinct
        proxy classes.

        Arguments
        - loader: the class loader to define the proxy class
        - interfaces: the list of interfaces for the proxy class
                 to implement
        - h: the invocation handler to dispatch method invocations to

        Returns
        - a proxy instance with the specified invocation handler of a
                 proxy class that is defined by the specified class loader
                 and that implements the specified interfaces

        Raises
        - IllegalArgumentException: if any of the <a href="#restrictions">
                 restrictions</a> on the parameters are violated
        - SecurityException: if a security manager, *s*, is present
                 and any of the following conditions is met:
                 
                 -  the given `loader` is `null` and
                      the caller's class loader is not `null` and the
                      invocation of SecurityManager.checkPermission
                      s.checkPermission with
                      `RuntimePermission("getClassLoader")` permission
                      denies access;
                 -  for each proxy interface, `intf`,
                      the caller's class loader is not the same as or an
                      ancestor of the class loader for `intf` and
                      invocation of SecurityManager.checkPackageAccess
                      s.checkPackageAccess() denies access to `intf`;
                 -  any of the given proxy interfaces is non-public and the
                      caller class is not in the same Package runtime package
                      as the non-public interface and the invocation of
                      SecurityManager.checkPermission s.checkPermission with
                      `ReflectPermission("newProxyInPackage.{package name`")}
                      permission denies access.
                 
        - NullPointerException: if the `interfaces` array
                 argument or any of its elements are `null`, or
                 if the invocation handler, `h`, is
                 `null`

        See
        - <a href=".membership">Package and Module Membership of Proxy Class</a>

        Unknown Tags
        - 9
        """
        ...


    @staticmethod
    def isProxyClass(cl: type[Any]) -> bool:
        """
        Returns True if the given class is a proxy class.

        Arguments
        - cl: the class to test

        Returns
        - `True` if the class is a proxy class and
                 `False` otherwise

        Raises
        - NullPointerException: if `cl` is `null`

        Unknown Tags
        - The reliability of this method is important for the ability
        to use it to make security decisions, so its implementation should
        not just test if the class in question extends `Proxy`.
        - 9
        """
        ...


    @staticmethod
    def getInvocationHandler(proxy: "Object") -> "InvocationHandler":
        """
        Returns the invocation handler for the specified proxy instance.

        Arguments
        - proxy: the proxy instance to return the invocation handler for

        Returns
        - the invocation handler for the proxy instance

        Raises
        - IllegalArgumentException: if the argument is not a
                 proxy instance
        - SecurityException: if a security manager, *s*, is present
                 and the caller's class loader is not the same as or an
                 ancestor of the class loader for the invocation handler
                 and invocation of SecurityManager.checkPackageAccess
                 s.checkPackageAccess() denies access to the invocation
                 handler's class.
        """
        ...



"""
Python module generated from Java source file org.bukkit.event.player.PlayerItemBreakEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.entity import Player
from org.bukkit.event import HandlerList
from org.bukkit.event.player import *
from org.bukkit.inventory import ItemStack
from typing import Any, Callable, Iterable, Tuple


class PlayerItemBreakEvent(PlayerEvent):
    """
    Fired when a player's item breaks (such as a shovel or flint and steel).
    
    After this event, the item's amount will be set to `item amount - 1`
    and its durability will be reset to 0.
    """

    def __init__(self, player: "Player", brokenItem: "ItemStack"):
        ...


    def getBrokenItem(self) -> "ItemStack":
        """
        Gets the item that broke

        Returns
        - The broken item
        """
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file com.google.common.util.concurrent.AbstractFuture

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.base import Strings
from com.google.common.util.concurrent import *
from com.google.common.util.concurrent.internal import InternalFutureFailureAccess
from com.google.common.util.concurrent.internal import InternalFutures
from com.google.errorprone.annotations import CanIgnoreReturnValue
from com.google.errorprone.annotations import ForOverride
from com.google.j2objc.annotations import ReflectionSupport
from java.lang.reflect import Field
from java.security import AccessController
from java.security import PrivilegedActionException
from java.security import PrivilegedExceptionAction
from java.util import Locale
from java.util.concurrent import CancellationException
from java.util.concurrent import ExecutionException
from java.util.concurrent import Executor
from java.util.concurrent import Future
from java.util.concurrent import ScheduledFuture
from java.util.concurrent import TimeUnit
from java.util.concurrent import TimeoutException
from java.util.concurrent.atomic import AtomicReferenceFieldUpdater
from java.util.concurrent.locks import LockSupport
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import Nullable
from sun.misc import Unsafe
from typing import Any, Callable, Iterable, Tuple


class AbstractFuture(InternalFutureFailureAccess, ListenableFuture):
    """
    An abstract implementation of ListenableFuture, intended for advanced users only. More
    common ways to create a `ListenableFuture` include instantiating a SettableFuture,
    submitting a task to a ListeningExecutorService, and deriving a `Future` from an
    existing one, typically using methods like Futures.transform(ListenableFuture,
    com.google.common.base.Function, java.util.concurrent.Executor) Futures.transform and Futures.catching(ListenableFuture, Class, com.google.common.base.Function,
    java.util.concurrent.Executor) Futures.catching.
    
    This class implements all methods in `ListenableFuture`. Subclasses should provide a way
    to set the result of the computation through the protected methods .set(Object), .setFuture(ListenableFuture) and .setException(Throwable). Subclasses may also override
    .afterDone(), which will be invoked automatically when the future completes. Subclasses
    should rarely override other methods.

    Author(s)
    - Luke Sandberg

    Since
    - 1.0
    """

    def get(self, timeout: int, unit: "TimeUnit") -> "V":
        """
        
        
        The default AbstractFuture implementation throws `InterruptedException` if the
        current thread is interrupted during the call, even if the value is already available.

        Raises
        - CancellationException: 
        """
        ...


    def get(self) -> "V":
        """
        
        
        The default AbstractFuture implementation throws `InterruptedException` if the
        current thread is interrupted during the call, even if the value is already available.

        Raises
        - CancellationException: 
        """
        ...


    def isDone(self) -> bool:
        ...


    def isCancelled(self) -> bool:
        ...


    def cancel(self, mayInterruptIfRunning: bool) -> bool:
        """
        
        
        If a cancellation attempt succeeds on a `Future` that had previously been .setFuture set asynchronously, then the cancellation will also be propagated to the delegate
        `Future` that was supplied in the `setFuture` call.
        
        Rather than override this method to perform additional cancellation work or cleanup,
        subclasses should override .afterDone, consulting .isCancelled and .wasInterrupted as necessary. This ensures that the work is done even if the future is
        cancelled without a call to `cancel`, such as by calling `setFuture(cancelledFuture)`.
        
        Beware of completing a future while holding a lock. Its listeners may do slow work or
        acquire other locks, risking deadlocks.
        """
        ...


    def addListener(self, listener: "Runnable", executor: "Executor") -> None:
        """
        Since
        - 10.0
        """
        ...


    def toString(self) -> str:
        ...



"""
Python module generated from Java source file org.bukkit.scoreboard.Criteria

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.base import Preconditions
from org.bukkit import Bukkit
from org.bukkit import Material
from org.bukkit import Statistic
from org.bukkit.Statistic import Type
from org.bukkit.entity import EntityType
from org.bukkit.scoreboard import *
from typing import Any, Callable, Iterable, Tuple


class Criteria:
    """
    Represents a scoreboard criteria, either custom or built-in to the Minecraft server, used to
    keep track of and manually or automatically change scores on a scoreboard.
    
    While this class outlines constants for standard criteria, see .statistic(Statistic)
    (and its overloads) to create instances for statistically-backed criteria.
    """

    DUMMY = Bukkit.getScoreboardCriteria("dummy")
    """
    The dummy criteria. Not changed by the server.
    """
    TRIGGER = Bukkit.getScoreboardCriteria("trigger")
    """
    The trigger criteria. Changed when a player runs the /trigger command for an objective.
    """
    DEATH_COUNT = Bukkit.getScoreboardCriteria("deathCount")
    """
    Increments automatically when a player dies.
    """
    PLAYER_KILL_COUNT = Bukkit.getScoreboardCriteria("playerKillCount")
    """
    Increments automatically when a player kills another player.
    """
    TOTAL_KILL_COUNT = Bukkit.getScoreboardCriteria("totalKillCount")
    """
    Increments automatically when a player kills another living entity.
    """
    HEALTH = Bukkit.getScoreboardCriteria("health")
    """
    Mirrors the player's health points (0 for no health, 20 for maximum default health).
    """
    FOOD = Bukkit.getScoreboardCriteria("food")
    """
    Mirrors the player's food points (0 for no food, 20 for maximum food).
    """
    AIR = Bukkit.getScoreboardCriteria("air")
    """
    Mirrors the player's air supply (0 for no air, 300 for maximum air).
    """
    ARMOR = Bukkit.getScoreboardCriteria("armor")
    """
    Mirrors the player's armor points (0 for no armor, 20 for maximum armor).
    """
    XP = Bukkit.getScoreboardCriteria("xp")
    """
    Mirrors the player's experience points.
    """
    LEVEL = Bukkit.getScoreboardCriteria("level")
    """
    Mirrors the player's experience level.
    """
    TEAM_KILL_BLACK = Bukkit.getScoreboardCriteria("teamkill.black")
    """
    Increments automatically when a player kills another player on the black team.
    """
    TEAM_KILL_DARK_BLUE = Bukkit.getScoreboardCriteria("teamkill.dark_blue")
    """
    Increments automatically when a player kills another player on the dark blue team.
    """
    TEAM_KILL_DARK_GREEN = Bukkit.getScoreboardCriteria("teamkill.dark_green")
    """
    Increments automatically when a player kills another player on the dark green team.
    """
    TEAM_KILL_DARK_AQUA = Bukkit.getScoreboardCriteria("teamkill.dark_aqua")
    """
    Increments automatically when a player kills another player on the dark aqua team.
    """
    TEAM_KILL_DARK_RED = Bukkit.getScoreboardCriteria("teamkill.dark_red")
    """
    Increments automatically when a player kills another player on the dark red team.
    """
    TEAM_KILL_DARK_PURPLE = Bukkit.getScoreboardCriteria("teamkill.dark_purple")
    """
    Increments automatically when a player kills another player on the dark purple team.
    """
    TEAM_KILL_GOLD = Bukkit.getScoreboardCriteria("teamkill.gold")
    """
    Increments automatically when a player kills another player on the gold team.
    """
    TEAM_KILL_GRAY = Bukkit.getScoreboardCriteria("teamkill.gray")
    """
    Increments automatically when a player kills another player on the gray team.
    """
    TEAM_KILL_DARK_GRAY = Bukkit.getScoreboardCriteria("teamkill.dark_gray")
    """
    Increments automatically when a player kills another player on the dark gray team.
    """
    TEAM_KILL_BLUE = Bukkit.getScoreboardCriteria("teamkill.blue")
    """
    Increments automatically when a player kills another player on the blue team.
    """
    TEAM_KILL_GREEN = Bukkit.getScoreboardCriteria("teamkill.green")
    """
    Increments automatically when a player kills another player on the green team.
    """
    TEAM_KILL_AQUA = Bukkit.getScoreboardCriteria("teamkill.aqua")
    """
    Increments automatically when a player kills another player on the aqua team.
    """
    TEAM_KILL_RED = Bukkit.getScoreboardCriteria("teamkill.red")
    """
    Increments automatically when a player kills another player on the red team.
    """
    TEAM_KILL_LIGHT_PURPLE = Bukkit.getScoreboardCriteria("teamkill.light_purple")
    """
    Increments automatically when a player kills another player on the light purple team.
    """
    TEAM_KILL_YELLOW = Bukkit.getScoreboardCriteria("teamkill.yellow")
    """
    Increments automatically when a player kills another player on the yellow team.
    """
    TEAM_KILL_WHITE = Bukkit.getScoreboardCriteria("teamkill.white")
    """
    Increments automatically when a player kills another player on the white team.
    """
    KILLED_BY_TEAM_BLACK = Bukkit.getScoreboardCriteria("killedByTeam.black")
    """
    Increments automatically when a player is killed by a player on the black team.
    """
    KILLED_BY_TEAM_DARK_BLUE = Bukkit.getScoreboardCriteria("killedByTeam.dark_blue")
    """
    Increments automatically when a player is killed by a player on the dark blue team.
    """
    KILLED_BY_TEAM_DARK_GREEN = Bukkit.getScoreboardCriteria("killedByTeam.dark_green")
    """
    Increments automatically when a player is killed by a player on the dark green team.
    """
    KILLED_BY_TEAM_DARK_AQUA = Bukkit.getScoreboardCriteria("killedByTeam.dark_aqua")
    """
    Increments automatically when a player is killed by a player on the dark aqua team.
    """
    KILLED_BY_TEAM_DARK_RED = Bukkit.getScoreboardCriteria("killedByTeam.dark_red")
    """
    Increments automatically when a player is killed by a player on the dark red team.
    """
    KILLED_BY_TEAM_DARK_PURPLE = Bukkit.getScoreboardCriteria("killedByTeam.dark_purple")
    """
    Increments automatically when a player is killed by a player on the dark purple team.
    """
    KILLED_BY_TEAM_GOLD = Bukkit.getScoreboardCriteria("killedByTeam.gold")
    """
    Increments automatically when a player is killed by a player on the gold team.
    """
    KILLED_BY_TEAM_GRAY = Bukkit.getScoreboardCriteria("killedByTeam.gray")
    """
    Increments automatically when a player is killed by a player on the gray team.
    """
    KILLED_BY_TEAM_DARK_GRAY = Bukkit.getScoreboardCriteria("killedByTeam.dark_gray")
    """
    Increments automatically when a player is killed by a player on the dark gray team.
    """
    KILLED_BY_TEAM_BLUE = Bukkit.getScoreboardCriteria("killedByTeam.blue")
    """
    Increments automatically when a player is killed by a player on the blue team.
    """
    KILLED_BY_TEAM_GREEN = Bukkit.getScoreboardCriteria("killedByTeam.green")
    """
    Increments automatically when a player is killed by a player on the green team.
    """
    KILLED_BY_TEAM_AQUA = Bukkit.getScoreboardCriteria("killedByTeam.aqua")
    """
    Increments automatically when a player is killed by a player on the aqua team.
    """
    KILLED_BY_TEAM_RED = Bukkit.getScoreboardCriteria("killedByTeam.red")
    """
    Increments automatically when a player is killed by a player on the red team.
    """
    KILLED_BY_TEAM_LIGHT_PURPLE = Bukkit.getScoreboardCriteria("killedByTeam.light_purple")
    """
    Increments automatically when a player is killed by a player on the light purple team.
    """
    KILLED_BY_TEAM_YELLOW = Bukkit.getScoreboardCriteria("killedByTeam.yellow")
    """
    Increments automatically when a player is killed by a player on the yellow team.
    """
    KILLED_BY_TEAM_WHITE = Bukkit.getScoreboardCriteria("killedByTeam.white")
    """
    Increments automatically when a player is killed by a player on the white team.
    """


    def getName(self) -> str:
        """
        Get the name of this criteria (its unique id).

        Returns
        - the name
        """
        ...


    def isReadOnly(self) -> bool:
        """
        Get whether or not this criteria is read only. If read only, scoreboards with this criteria
        cannot have their scores changed.

        Returns
        - True if read only, False otherwise
        """
        ...


    def getDefaultRenderType(self) -> "RenderType":
        """
        Get the RenderType used by default for this criteria.

        Returns
        - the default render type
        """
        ...


    @staticmethod
    def statistic(statistic: "Statistic", material: "Material") -> "Criteria":
        """
        Get a Criteria for the specified statistic pertaining to blocks or items.
        
        This method expects a Statistic of Type.BLOCK or Type.ITEM and the
        Material matching said type (e.g. BLOCK statistics require materials where
        Material.isBlock() is True). This acts as a convenience to create more complex
        compound criteria such as those that increment on block breaks, or item uses. An example
        would be `Criteria.statistic(Statistic.CRAFT_ITEM, Material.STICK)`, returning a
        Criteria representing "minecraft.crafted:minecraft.stick" which will increment when the
        player crafts a stick.
        
        If the provided statistic does not require additional data, .statistic(Statistic)
        is called and returned instead.
        
        This method provides no guarantee that any given criteria exists on the vanilla server.

        Arguments
        - statistic: the statistic for which to get a criteria
        - material: the relevant material

        Returns
        - the criteria

        Raises
        - IllegalArgumentException: if Statistic.getType() is anything other than
        Type.BLOCK or Type.ITEM
        - IllegalArgumentException: if Statistic.getType() is Type.BLOCK, but
        Material.isBlock() is False
        - IllegalArgumentException: if Statistic.getType() is Type.ITEM, but
        Material.isItem() is False
        """
        ...


    @staticmethod
    def statistic(statistic: "Statistic", entityType: "EntityType") -> "Criteria":
        """
        Get a Criteria for the specified statistic pertaining to an entity type.
        
        This method expects a Statistic of Type.ENTITY. This acts as a convenience
        to create more complex compound criteria such as being killed by a specific entity type.
        An example would be `Criteria.statistic(Statistic.KILL_ENTITY, EntityType.CREEPER)`,
        returning a Criteria representing "minecraft.killed:minecraft.creeper" which will increment
        when the player kills a creepers.
        
        If the provided statistic does not require additional data, .statistic(Statistic)
        is called and returned instead.
        
        This method provides no guarantee that any given criteria exists on the vanilla server.

        Arguments
        - statistic: the statistic for which to get a criteria
        - entityType: the relevant entity type

        Returns
        - the criteria

        Raises
        - IllegalArgumentException: if Statistic.getType() is not Type.ENTITY
        """
        ...


    @staticmethod
    def statistic(statistic: "Statistic") -> "Criteria":
        """
        Get a Criteria for the specified statistic.
        
        An example would be `Criteria.statistic(Statistic.FLY_ONE_CM)`, returning a Criteria
        representing "minecraft.custom:minecraft.aviate_one_cm" which will increment when the player
        flies with an elytra.
        
        This method provides no guarantee that any given criteria exists on the vanilla server. All
        statistics are accepted, however some may not operate as expected if additional data is
        required. For block/item-related statistics, see .statistic(Statistic, Material),
        and for entity-related statistics, see .statistic(Statistic, EntityType)

        Arguments
        - statistic: the statistic for which to get a criteria

        Returns
        - the criteria
        """
        ...


    @staticmethod
    def create(name: str) -> "Criteria":
        """
        Get (or create) a new Criteria by its name.

        Arguments
        - name: the criteria name

        Returns
        - the created criteria
        """
        ...



"""
Python module generated from Java source file org.yaml.snakeyaml.constructor.AbstractConstruct

Java source file obtained from artifact snakeyaml version 2.2

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.yaml.snakeyaml.constructor import *
from org.yaml.snakeyaml.error import YAMLException
from org.yaml.snakeyaml.nodes import Node
from typing import Any, Callable, Iterable, Tuple


class AbstractConstruct(Construct):
    """
    Because recursive structures are not very common we provide a way to save some typing when
    extending a constructor
    """

    def construct2ndStep(self, node: "Node", data: "Object") -> None:
        """
        Fail with a reminder to provide the seconds step for a recursive structure

        See
        - org.yaml.snakeyaml.constructor.Construct.construct2ndStep(org.yaml.snakeyaml.nodes.Node,
             java.lang.Object)
        """
        ...



"""
Python module generated from Java source file org.bukkit.event.player.PlayerExpCooldownChangeEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from org.bukkit.entity import Player
from org.bukkit.event import HandlerList
from org.bukkit.event.player import *
from typing import Any, Callable, Iterable, Tuple


class PlayerExpCooldownChangeEvent(PlayerEvent):
    """
    Called when a player's experience cooldown changes.
    """

    def __init__(self, player: "Player", newcooldown: int, reason: "ChangeReason"):
        ...


    def getReason(self) -> "ChangeReason":
        """
        Gets the reason for the change.

        Returns
        - The reason for the change
        """
        ...


    def getNewCooldown(self) -> int:
        """
        Gets the new cooldown for the player.

        Returns
        - The new cooldown

        See
        - Player.getExpCooldown()
        """
        ...


    def setNewCooldown(self, newCooldown: int) -> None:
        """
        Sets the new cooldown for the player.

        Arguments
        - newCooldown: The new cooldown to set

        See
        - Player.setExpCooldown(int)
        """
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...


    class ChangeReason(Enum):

        PICKUP_ORB = 0
        """
        The cooldown was set by picking up an experience orb.
        """
        PLUGIN = 1
        """
        The cooldown was set by a plugin.

        See
        - Player.setExpCooldown(int)
        """



"""
Python module generated from Java source file com.google.common.cache.AbstractLoadingCache

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtIncompatible
from com.google.common.cache import *
from com.google.common.collect import ImmutableMap
from com.google.common.collect import Maps
from com.google.common.util.concurrent import UncheckedExecutionException
from com.google.errorprone.annotations import CanIgnoreReturnValue
from java.util.concurrent import Callable
from java.util.concurrent import ExecutionException
from typing import Any, Callable, Iterable, Tuple


class AbstractLoadingCache(AbstractCache, LoadingCache):
    """
    This class provides a skeletal implementation of the `Cache` interface to minimize the
    effort required to implement this interface.
    
    To implement a cache, the programmer needs only to extend this class and provide an
    implementation for the .get(Object) and .getIfPresent methods. .getUnchecked, .get(Object, Callable), and .getAll are implemented in terms of
    `get`; .getAllPresent is implemented in terms of `getIfPresent`; .putAll is implemented in terms of .put, .invalidateAll(Iterable) is implemented
    in terms of .invalidate. The method .cleanUp is a no-op. All other methods throw
    an UnsupportedOperationException.

    Author(s)
    - Charles Fry

    Since
    - 11.0
    """

    def getUnchecked(self, key: "K") -> "V":
        ...


    def getAll(self, keys: Iterable["K"]) -> "ImmutableMap"["K", "V"]:
        ...


    def apply(self, key: "K") -> "V":
        ...


    def refresh(self, key: "K") -> None:
        ...



"""
Python module generated from Java source file com.google.common.collect.Interners

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.annotations import VisibleForTesting
from com.google.common.base import Equivalence
from com.google.common.base import Function
from com.google.common.collect import *
from com.google.common.collect.MapMaker import Dummy
from com.google.common.collect.MapMakerInternalMap import InternalEntry
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class Interners:
    """
    Contains static methods pertaining to instances of Interner.

    Author(s)
    - Kevin Bourrillion

    Since
    - 3.0
    """

    @staticmethod
    def newBuilder() -> "InternerBuilder":
        """
        Returns a fresh InternerBuilder instance.
        """
        ...


    @staticmethod
    def newStrongInterner() -> "Interner"["E"]:
        """
        Returns a new thread-safe interner which retains a strong reference to each instance it has
        interned, thus preventing these instances from being garbage-collected. If this retention is
        acceptable, this implementation may perform better than .newWeakInterner.
        """
        ...


    @staticmethod
    def newWeakInterner() -> "Interner"["E"]:
        """
        Returns a new thread-safe interner which retains a weak reference to each instance it has
        interned, and so does not prevent these instances from being garbage-collected. This most
        likely does not perform as well as .newStrongInterner, but is the best alternative when
        the memory usage of that implementation is unacceptable.
        """
        ...


    @staticmethod
    def asFunction(interner: "Interner"["E"]) -> "Function"["E", "E"]:
        """
        Returns a function that delegates to the Interner.intern method of the given interner.

        Since
        - 8.0
        """
        ...


    class InternerBuilder:
        """
        Builder for Interner instances.

        Since
        - 21.0
        """

        def strong(self) -> "InternerBuilder":
            """
            Instructs the InternerBuilder to build a strong interner.

            See
            - Interners.newStrongInterner()
            """
            ...


        def weak(self) -> "InternerBuilder":
            """
            Instructs the InternerBuilder to build a weak interner.

            See
            - Interners.newWeakInterner()
            """
            ...


        def concurrencyLevel(self, concurrencyLevel: int) -> "InternerBuilder":
            """
            Sets the concurrency level that will be used by the to-be-built Interner.

            See
            - MapMaker.concurrencyLevel(int)
            """
            ...


        def build(self) -> "Interner"["E"]:
            ...



"""
Python module generated from Java source file com.google.common.base.Suppliers

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.annotations import VisibleForTesting
from com.google.common.base import *
from java.io import IOException
from java.io import ObjectInputStream
from java.io import Serializable
from java.time import Duration
from java.util.concurrent import TimeUnit
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class Suppliers:
    """
    Useful suppliers.
    
    All methods return serializable suppliers as long as they're given serializable parameters.

    Author(s)
    - Harry Heymann

    Since
    - 2.0
    """

    @staticmethod
    def compose(function: "Function"["F", "T"], supplier: "Supplier"["F"]) -> "Supplier"["T"]:
        """
        Returns a new supplier which is the composition of the provided function and supplier. In other
        words, the new supplier's value will be computed by retrieving the value from `supplier`,
        and then applying `function` to that value. Note that the resulting supplier will not
        call `supplier` or invoke `function` until it is called.
        """
        ...


    @staticmethod
    def memoize(delegate: "Supplier"["T"]) -> "Supplier"["T"]:
        """
        Returns a supplier which caches the instance retrieved during the first call to `get()`
        and returns that value on subsequent calls to `get()`. See: <a
        href="http://en.wikipedia.org/wiki/Memoization">memoization</a>
        
        The returned supplier is thread-safe. The delegate's `get()` method will be invoked at
        most once unless the underlying `get()` throws an exception. The supplier's serialized
        form does not contain the cached value, which will be recalculated when `get()` is called
        on the deserialized instance.
        
        When the underlying delegate throws an exception then this memoizing supplier will keep
        delegating calls until it returns valid data.
        
        If `delegate` is an instance created by an earlier call to `memoize`, it is
        returned directly.
        """
        ...


    @staticmethod
    def memoizeWithExpiration(delegate: "Supplier"["T"], duration: int, unit: "TimeUnit") -> "Supplier"["T"]:
        """
        Returns a supplier that caches the instance supplied by the delegate and removes the cached
        value after the specified time has passed. Subsequent calls to `get()` return the cached
        value if the expiration time has not passed. After the expiration time, a new value is
        retrieved, cached, and returned. See: <a
        href="http://en.wikipedia.org/wiki/Memoization">memoization</a>
        
        The returned supplier is thread-safe. The supplier's serialized form does not contain the
        cached value, which will be recalculated when `get()` is called on the reserialized
        instance. The actual memoization does not happen when the underlying delegate throws an
        exception.
        
        When the underlying delegate throws an exception then this memoizing supplier will keep
        delegating calls until it returns valid data.

        Arguments
        - duration: the length of time after a value is created that it should stop being returned
            by subsequent `get()` calls
        - unit: the unit that `duration` is expressed in

        Raises
        - IllegalArgumentException: if `duration` is not positive

        Since
        - 2.0
        """
        ...


    @staticmethod
    def memoizeWithExpiration(delegate: "Supplier"["T"], duration: "Duration") -> "Supplier"["T"]:
        """
        Returns a supplier that caches the instance supplied by the delegate and removes the cached
        value after the specified time has passed. Subsequent calls to `get()` return the cached
        value if the expiration time has not passed. After the expiration time, a new value is
        retrieved, cached, and returned. See: <a
        href="http://en.wikipedia.org/wiki/Memoization">memoization</a>
        
        The returned supplier is thread-safe. The supplier's serialized form does not contain the
        cached value, which will be recalculated when `get()` is called on the reserialized
        instance. The actual memoization does not happen when the underlying delegate throws an
        exception.
        
        When the underlying delegate throws an exception then this memoizing supplier will keep
        delegating calls until it returns valid data.

        Arguments
        - duration: the length of time after a value is created that it should stop being returned
            by subsequent `get()` calls

        Raises
        - IllegalArgumentException: if `duration` is not positive

        Since
        - 33.1.0
        """
        ...


    @staticmethod
    def ofInstance(instance: "T") -> "Supplier"["T"]:
        """
        Returns a supplier that always supplies `instance`.
        """
        ...


    @staticmethod
    def synchronizedSupplier(delegate: "Supplier"["T"]) -> "Supplier"["T"]:
        """
        Returns a supplier whose `get()` method synchronizes on `delegate` before calling
        it, making it thread-safe.
        """
        ...


    @staticmethod
    def supplierFunction() -> "Function"["Supplier"["T"], "T"]:
        """
        Returns a function that accepts a supplier and returns the result of invoking Supplier.get on that supplier.
        
        **Java 8+ users:** use the method reference `Supplier::get` instead.

        Since
        - 8.0
        """
        ...



"""
Python module generated from Java source file org.bukkit.event.hanging.HangingBreakByEntityEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.entity import Entity
from org.bukkit.entity import Hanging
from org.bukkit.event.hanging import *
from typing import Any, Callable, Iterable, Tuple


class HangingBreakByEntityEvent(HangingBreakEvent):
    """
    Triggered when a hanging entity is removed by an entity
    """

    def __init__(self, hanging: "Hanging", remover: "Entity"):
        ...


    def __init__(self, hanging: "Hanging", remover: "Entity", cause: "HangingBreakEvent.RemoveCause"):
        ...


    def getRemover(self) -> "Entity":
        """
        Gets the entity that removed the hanging entity.
        May be null, for example when broken by an explosion.

        Returns
        - the entity that removed the hanging entity
        """
        ...



"""
Python module generated from Java source file com.google.common.hash.AbstractHasher

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.base import Preconditions
from com.google.common.hash import *
from com.google.errorprone.annotations import CanIgnoreReturnValue
from java.nio.charset import Charset
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class AbstractHasher(Hasher):
    """
    An abstract implementation of Hasher, which only requires subtypes to implement .putByte. Subtypes may provide more efficient implementations, however.

    Author(s)
    - Dimitris Andreou
    """

    def putBoolean(self, b: bool) -> "Hasher":
        ...


    def putDouble(self, d: float) -> "Hasher":
        ...


    def putFloat(self, f: float) -> "Hasher":
        ...


    def putUnencodedChars(self, charSequence: "CharSequence") -> "Hasher":
        ...


    def putString(self, charSequence: "CharSequence", charset: "Charset") -> "Hasher":
        ...


    def putBytes(self, bytes: list[int]) -> "Hasher":
        ...


    def putBytes(self, bytes: list[int], off: int, len: int) -> "Hasher":
        ...


    def putBytes(self, b: "ByteBuffer") -> "Hasher":
        ...


    def putShort(self, s: int) -> "Hasher":
        ...


    def putInt(self, i: int) -> "Hasher":
        ...


    def putLong(self, l: int) -> "Hasher":
        ...


    def putChar(self, c: str) -> "Hasher":
        ...


    def putObject(self, instance: "T", funnel: "Funnel"["T"]) -> "Hasher":
        ...



"""
Python module generated from Java source file org.bukkit.scoreboard.NameTagVisibility

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from org.bukkit.scoreboard import *
from typing import Any, Callable, Iterable, Tuple


class NameTagVisibility(Enum):
    """
    Deprecated
    - replaced by Team.OptionStatus
    """

    ALWAYS = 0
    """
    Always show the player's nametag.
    """
    NEVER = 1
    """
    Never show the player's nametag.
    """
    HIDE_FOR_OTHER_TEAMS = 2
    """
    Show the player's nametag only to his own team members.
    """
    HIDE_FOR_OWN_TEAM = 3
    """
    Show the player's nametag only to members of other teams.
    """



"""
Python module generated from Java source file org.bukkit.event.block.BlockSpreadEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.block import Block
from org.bukkit.block import BlockState
from org.bukkit.event import HandlerList
from org.bukkit.event.block import *
from typing import Any, Callable, Iterable, Tuple


class BlockSpreadEvent(BlockFormEvent):
    """
    Called when a block spreads based on world conditions.
    
    Use BlockFormEvent to catch blocks that "randomly" form instead of
    actually spread.
    
    Examples:
    
    - Mushrooms spreading.
    - Fire spreading.
    
    
    If a Block Spread event is cancelled, the block will not spread.

    See
    - BlockFormEvent
    """

    def __init__(self, block: "Block", source: "Block", newState: "BlockState"):
        ...


    def getSource(self) -> "Block":
        """
        Gets the source block involved in this event.

        Returns
        - the Block for the source block involved in this event.
        """
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file java.util.WeakHashMap

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.lang.ref import ReferenceQueue
from java.lang.ref import WeakReference
from java.util import *
from java.util.function import BiConsumer
from java.util.function import BiFunction
from java.util.function import Consumer
from typing import Any, Callable, Iterable, Tuple


class WeakHashMap(AbstractMap, Map):
    """
    Hash table based implementation of the `Map` interface, with
    *weak keys*.
    An entry in a `WeakHashMap` will automatically be removed when
    its key is no longer in ordinary use.  More precisely, the presence of a
    mapping for a given key will not prevent the key from being discarded by the
    garbage collector, that is, made finalizable, finalized, and then reclaimed.
    When a key has been discarded its entry is effectively removed from the map,
    so this class behaves somewhat differently from other `Map`
    implementations.
    
     Both null values and the null key are supported. This class has
    performance characteristics similar to those of the `HashMap`
    class, and has the same efficiency parameters of *initial capacity*
    and *load factor*.
    
     Like most collection classes, this class is not synchronized.
    A synchronized `WeakHashMap` may be constructed using the
    Collections.synchronizedMap Collections.synchronizedMap
    method.
    
     This class is intended primarily for use with key objects whose
    `equals` methods test for object identity using the
    `==` operator.  Once such a key is discarded it can never be
    recreated, so it is impossible to do a lookup of that key in a
    `WeakHashMap` at some later time and be surprised that its entry
    has been removed.  This class will work perfectly well with key objects
    whose `equals` methods are not based upon object identity, such
    as `String` instances.  With such recreatable key objects,
    however, the automatic removal of `WeakHashMap` entries whose
    keys have been discarded may prove to be confusing.
    
     The behavior of the `WeakHashMap` class depends in part upon
    the actions of the garbage collector, so several familiar (though not
    required) `Map` invariants do not hold for this class.  Because
    the garbage collector may discard keys at any time, a
    `WeakHashMap` may behave as though an unknown thread is silently
    removing entries.  In particular, even if you synchronize on a
    `WeakHashMap` instance and invoke none of its mutator methods, it
    is possible for the `size` method to return smaller values over
    time, for the `isEmpty` method to return `False` and
    then `True`, for the `containsKey` method to return
    `True` and later `False` for a given key, for the
    `get` method to return a value for a given key but later return
    `null`, for the `put` method to return
    `null` and the `remove` method to return
    `False` for a key that previously appeared to be in the map, and
    for successive examinations of the key set, the value collection, and
    the entry set to yield successively smaller numbers of elements.
    
     Each key object in a `WeakHashMap` is stored indirectly as
    the referent of a weak reference.  Therefore a key will automatically be
    removed only after the weak references to it, both inside and outside of the
    map, have been cleared by the garbage collector.
    
     <strong>Implementation note:</strong> The value objects in a
    `WeakHashMap` are held by ordinary strong references.  Thus care
    should be taken to ensure that value objects do not strongly refer to their
    own keys, either directly or indirectly, since that will prevent the keys
    from being discarded.  Note that a value object may refer indirectly to its
    key via the `WeakHashMap` itself; that is, a value object may
    strongly refer to some other key object whose associated value object, in
    turn, strongly refers to the key of the first value object.  If the values
    in the map do not rely on the map holding strong references to them, one way
    to deal with this is to wrap values themselves within
    `WeakReferences` before
    inserting, as in: `m.put(key, new WeakReference(value))`,
    and then unwrapping upon each `get`.
    
    The iterators returned by the `iterator` method of the collections
    returned by all of this class's "collection view methods" are
    *fail-fast*: if the map is structurally modified at any time after the
    iterator is created, in any way except through the iterator's own
    `remove` method, the iterator will throw a ConcurrentModificationException.  Thus, in the face of concurrent
    modification, the iterator fails quickly and cleanly, rather than risking
    arbitrary, non-deterministic behavior at an undetermined time in the future.
    
    Note that the fail-fast behavior of an iterator cannot be guaranteed
    as it is, generally speaking, impossible to make any hard guarantees in the
    presence of unsynchronized concurrent modification.  Fail-fast iterators
    throw `ConcurrentModificationException` on a best-effort basis.
    Therefore, it would be wrong to write a program that depended on this
    exception for its correctness:  *the fail-fast behavior of iterators
    should be used only to detect bugs.*
    
    This class is a member of the
    <a href="/java.base/java/util/package-summary.html#CollectionsFramework">
    Java Collections Framework</a>.
    
    Type `<K>`: the type of keys maintained by this map
    
    Type `<V>`: the type of mapped values

    Author(s)
    - Mark Reinhold

    See
    - java.lang.ref.WeakReference

    Since
    - 1.2
    """

    def __init__(self, initialCapacity: int, loadFactor: float):
        """
        Constructs a new, empty `WeakHashMap` with the given initial
        capacity and the given load factor.

        Arguments
        - initialCapacity: The initial capacity of the `WeakHashMap`
        - loadFactor: The load factor of the `WeakHashMap`

        Raises
        - IllegalArgumentException: if the initial capacity is negative,
                or if the load factor is nonpositive.
        """
        ...


    def __init__(self, initialCapacity: int):
        """
        Constructs a new, empty `WeakHashMap` with the given initial
        capacity and the default load factor (0.75).

        Arguments
        - initialCapacity: The initial capacity of the `WeakHashMap`

        Raises
        - IllegalArgumentException: if the initial capacity is negative
        """
        ...


    def __init__(self):
        """
        Constructs a new, empty `WeakHashMap` with the default initial
        capacity (16) and load factor (0.75).
        """
        ...


    def __init__(self, m: dict["K", "V"]):
        """
        Constructs a new `WeakHashMap` with the same mappings as the
        specified map.  The `WeakHashMap` is created with the default
        load factor (0.75) and an initial capacity sufficient to hold the
        mappings in the specified map.

        Arguments
        - m: the map whose mappings are to be placed in this map

        Raises
        - NullPointerException: if the specified map is null

        Since
        - 1.3
        """
        ...


    def size(self) -> int:
        """
        Returns the number of key-value mappings in this map.
        This result is a snapshot, and may not reflect unprocessed
        entries that will be removed before next attempted access
        because they are no longer referenced.
        """
        ...


    def isEmpty(self) -> bool:
        """
        Returns `True` if this map contains no key-value mappings.
        This result is a snapshot, and may not reflect unprocessed
        entries that will be removed before next attempted access
        because they are no longer referenced.
        """
        ...


    def get(self, key: "Object") -> "V":
        """
        Returns the value to which the specified key is mapped,
        or `null` if this map contains no mapping for the key.
        
        More formally, if this map contains a mapping from a key
        `k` to a value `v` such that
        `Objects.equals(key, k)`,
        then this method returns `v`; otherwise
        it returns `null`.  (There can be at most one such mapping.)
        
        A return value of `null` does not *necessarily*
        indicate that the map contains no mapping for the key; it's also
        possible that the map explicitly maps the key to `null`.
        The .containsKey containsKey operation may be used to
        distinguish these two cases.

        See
        - .put(Object, Object)
        """
        ...


    def containsKey(self, key: "Object") -> bool:
        """
        Returns `True` if this map contains a mapping for the
        specified key.

        Arguments
        - key: The key whose presence in this map is to be tested

        Returns
        - `True` if there is a mapping for `key`;
                `False` otherwise
        """
        ...


    def put(self, key: "K", value: "V") -> "V":
        """
        Associates the specified value with the specified key in this map.
        If the map previously contained a mapping for this key, the old
        value is replaced.

        Arguments
        - key: key with which the specified value is to be associated.
        - value: value to be associated with the specified key.

        Returns
        - the previous value associated with `key`, or
                `null` if there was no mapping for `key`.
                (A `null` return can also indicate that the map
                previously associated `null` with `key`.)
        """
        ...


    def putAll(self, m: dict["K", "V"]) -> None:
        """
        Copies all of the mappings from the specified map to this map.
        These mappings will replace any mappings that this map had for any
        of the keys currently in the specified map.

        Arguments
        - m: mappings to be stored in this map.

        Raises
        - NullPointerException: if the specified map is null.
        """
        ...


    def remove(self, key: "Object") -> "V":
        """
        Removes the mapping for a key from this weak hash map if it is present.
        More formally, if this map contains a mapping from key `k` to
        value `v` such that `(key==null ?  k==null :
        key.equals(k))`, that mapping is removed.  (The map can contain
        at most one such mapping.)
        
        Returns the value to which this map previously associated the key,
        or `null` if the map contained no mapping for the key.  A
        return value of `null` does not *necessarily* indicate
        that the map contained no mapping for the key; it's also possible
        that the map explicitly mapped the key to `null`.
        
        The map will not contain a mapping for the specified key once the
        call returns.

        Arguments
        - key: key whose mapping is to be removed from the map

        Returns
        - the previous value associated with `key`, or
                `null` if there was no mapping for `key`
        """
        ...


    def clear(self) -> None:
        """
        Removes all of the mappings from this map.
        The map will be empty after this call returns.
        """
        ...


    def containsValue(self, value: "Object") -> bool:
        """
        Returns `True` if this map maps one or more keys to the
        specified value.

        Arguments
        - value: value whose presence in this map is to be tested

        Returns
        - `True` if this map maps one or more keys to the
                specified value
        """
        ...


    def keySet(self) -> set["K"]:
        """
        Returns a Set view of the keys contained in this map.
        The set is backed by the map, so changes to the map are
        reflected in the set, and vice-versa.  If the map is modified
        while an iteration over the set is in progress (except through
        the iterator's own `remove` operation), the results of
        the iteration are undefined.  The set supports element removal,
        which removes the corresponding mapping from the map, via the
        `Iterator.remove`, `Set.remove`,
        `removeAll`, `retainAll`, and `clear`
        operations.  It does not support the `add` or `addAll`
        operations.
        """
        ...


    def values(self) -> Iterable["V"]:
        """
        Returns a Collection view of the values contained in this map.
        The collection is backed by the map, so changes to the map are
        reflected in the collection, and vice-versa.  If the map is
        modified while an iteration over the collection is in progress
        (except through the iterator's own `remove` operation),
        the results of the iteration are undefined.  The collection
        supports element removal, which removes the corresponding
        mapping from the map, via the `Iterator.remove`,
        `Collection.remove`, `removeAll`,
        `retainAll` and `clear` operations.  It does not
        support the `add` or `addAll` operations.
        """
        ...


    def entrySet(self) -> set["Map.Entry"["K", "V"]]:
        """
        Returns a Set view of the mappings contained in this map.
        The set is backed by the map, so changes to the map are
        reflected in the set, and vice-versa.  If the map is modified
        while an iteration over the set is in progress (except through
        the iterator's own `remove` operation, or through the
        `setValue` operation on a map entry returned by the
        iterator) the results of the iteration are undefined.  The set
        supports element removal, which removes the corresponding
        mapping from the map, via the `Iterator.remove`,
        `Set.remove`, `removeAll`, `retainAll` and
        `clear` operations.  It does not support the
        `add` or `addAll` operations.
        """
        ...


    def forEach(self, action: "BiConsumer"["K", "V"]) -> None:
        ...


    def replaceAll(self, function: "BiFunction"["K", "V", "V"]) -> None:
        ...



"""
Python module generated from Java source file com.google.gson.internal.ObjectConstructor

Java source file obtained from artifact gson version 2.11.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.gson.internal import *
from typing import Any, Callable, Iterable, Tuple


class ObjectConstructor:
    """
    Defines a generic object construction factory. The purpose of this class is to construct a
    default instance of a class that can be used for object navigation while deserialization from its
    JSON representation.

    Author(s)
    - Joel Leitch
    """

    def construct(self) -> "T":
        """
        Returns a new instance.
        """
        ...



from .UrlEscapers import UrlEscapers
from .PercentEscaper import PercentEscaper
from .ParametricNullness import ParametricNullness
from .MediaType import MediaType
from .InternetDomainName import InternetDomainName
from .InetAddresses import InetAddresses
from .HttpHeaders import HttpHeaders
from .HostSpecifier import HostSpecifier
from .HostAndPort import HostAndPort
from .ElementTypesAreNonnullByDefault import ElementTypesAreNonnullByDefault


"""
Python module generated from Java source file com.google.common.reflect.TypeVisitor

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.collect import Sets
from com.google.common.reflect import *
from java.lang.reflect import GenericArrayType
from java.lang.reflect import ParameterizedType
from java.lang.reflect import Type
from java.lang.reflect import TypeVariable
from java.lang.reflect import WildcardType
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class TypeVisitor:
    """
    Based on what a Type is, dispatch it to the corresponding `visit*` method. By
    default, no recursion is done for type arguments or type bounds. But subclasses can opt to do
    recursion by calling .visit for any `Type` while visitation is in progress. For
    example, this can be used to reject wildcards or type variables contained in a type as in:
    
    ````new TypeVisitor() {
      protected void visitParameterizedType(ParameterizedType t) {
        visit(t.getOwnerType());
        visit(t.getActualTypeArguments());`
      protected void visitGenericArrayType(GenericArrayType t) {
        visit(t.getGenericComponentType());
      }
      protected void visitTypeVariable(TypeVariable<?> t) {
        throw new IllegalArgumentException("Cannot contain type variable.");
      }
      protected void visitWildcardType(WildcardType t) {
        throw new IllegalArgumentException("Cannot contain wildcard type.");
      }
    }.visit(type);
    }```
    
    One `Type` is visited at most once. The second time the same type is visited, it's
    ignored by .visit. This avoids infinite recursion caused by recursive type bounds.
    
    This class is *not* thread safe.

    Author(s)
    - Ben Yu
    """

    def visit(self, *types: Tuple["Type", ...]) -> None:
        """
        Visits the given types. Null types are ignored. This allows subclasses to call `visit(parameterizedType.getOwnerType())` safely without having to check nulls.
        """
        ...



"""
Python module generated from Java source file org.bukkit.entity.PiglinAbstract

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.entity import *
from typing import Any, Callable, Iterable, Tuple


class PiglinAbstract(Monster, Ageable):
    """
    Piglin / Piglin Brute.
    """

    def isImmuneToZombification(self) -> bool:
        """
        Gets whether the piglin is immune to zombification.

        Returns
        - Whether the piglin is immune to zombification
        """
        ...


    def setImmuneToZombification(self, flag: bool) -> None:
        """
        Sets whether the piglin is immune to zombification.

        Arguments
        - flag: Whether the piglin is immune to zombification
        """
        ...


    def getConversionTime(self) -> int:
        """
        Gets the amount of ticks until this entity will be converted to a
        Zombified Piglin.
        
        When this reaches 300, the entity will be converted.

        Returns
        - conversion time

        Raises
        - IllegalStateException: if .isConverting() is False.
        """
        ...


    def setConversionTime(self, time: int) -> None:
        """
        Sets the amount of ticks until this entity will be converted to a
        Zombified Piglin.
        
        When this reaches 0, the entity will be converted. A value of less than 0
        will stop the current conversion process without converting the current
        entity.

        Arguments
        - time: new conversion time
        """
        ...


    def isConverting(self) -> bool:
        """
        Get if this entity is in the process of converting to a Zombified Piglin.

        Returns
        - conversion status
        """
        ...


    def isBaby(self) -> bool:
        """
        Gets whether the piglin is a baby

        Returns
        - Whether the piglin is a baby

        Deprecated
        - see Ageable.isAdult()
        """
        ...


    def setBaby(self, flag: bool) -> None:
        """
        Sets whether the piglin is a baby

        Arguments
        - flag: Whether the piglin is a baby

        Deprecated
        - see Ageable.setBaby() and Ageable.setAdult()
        """
        ...



"""
Python module generated from Java source file org.bukkit.event.block.BlockRedstoneEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.block import Block
from org.bukkit.event import HandlerList
from org.bukkit.event.block import *
from typing import Any, Callable, Iterable, Tuple


class BlockRedstoneEvent(BlockEvent):
    """
    Called when a redstone current changes
    """

    def __init__(self, block: "Block", oldCurrent: int, newCurrent: int):
        ...


    def getOldCurrent(self) -> int:
        """
        Gets the old current of this block

        Returns
        - The previous current
        """
        ...


    def getNewCurrent(self) -> int:
        """
        Gets the new current of this block

        Returns
        - The new current
        """
        ...


    def setNewCurrent(self, newCurrent: int) -> None:
        """
        Sets the new current of this block

        Arguments
        - newCurrent: The new current to set
        """
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file com.google.common.reflect.Parameter

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.collect import FluentIterable
from com.google.common.collect import ImmutableList
from com.google.common.reflect import *
from java.lang.reflect import AnnotatedElement
from java.lang.reflect import AnnotatedType
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class Parameter(AnnotatedElement):
    """
    Represents a method or constructor parameter.

    Author(s)
    - Ben Yu

    Since
    - 14.0
    """

    def getType(self) -> "TypeToken"[Any]:
        """
        Returns the type of the parameter.
        """
        ...


    def getDeclaringInvokable(self) -> "Invokable"[Any, Any]:
        """
        Returns the Invokable that declares this parameter.
        """
        ...


    def isAnnotationPresent(self, annotationType: type["Annotation"]) -> bool:
        ...


    def getAnnotation(self, annotationType: type["A"]) -> "A":
        ...


    def getAnnotations(self) -> list["Annotation"]:
        ...


    def getAnnotationsByType(self, annotationType: type["A"]) -> list["A"]:
        """
        Since
        - 18.0
        """
        ...


    def getDeclaredAnnotations(self) -> list["Annotation"]:
        """
        Since
        - 18.0
        """
        ...


    def getDeclaredAnnotation(self, annotationType: type["A"]) -> "A":
        """
        Since
        - 18.0
        """
        ...


    def getDeclaredAnnotationsByType(self, annotationType: type["A"]) -> list["A"]:
        """
        Since
        - 18.0
        """
        ...


    def getAnnotatedType(self) -> "AnnotatedType":
        """
        Returns the AnnotatedType of the parameter.

        Since
        - 25.1 for guava-jre
        """
        ...


    def equals(self, obj: "Object") -> bool:
        ...


    def hashCode(self) -> int:
        ...


    def toString(self) -> str:
        ...



"""
Python module generated from Java source file com.google.common.collect.ComparisonChain

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.collect import *
from com.google.common.primitives import Booleans
from java.util import Comparator
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class ComparisonChain:
    """
    A utility for performing a chained comparison statement. **Note:** Java 8+ users should
    generally prefer the methods in Comparator; see <a href="#java8">below</a>.
    
    Example usage of `ComparisonChain`:
    
    ````public int compareTo(Foo that) {
      return ComparisonChain.start()
          .compare(this.aString, that.aString)
          .compare(this.anInt, that.anInt)
          .compare(this.anEnum, that.anEnum, Ordering.natural().nullsLast())
          .result();`
    }```
    
    The value of this expression will have the same sign as the *first nonzero* comparison
    result in the chain, or will be zero if every comparison result was zero.
    
    **Note:** `ComparisonChain` instances are **immutable**. For this utility to work
    correctly, calls must be chained as illustrated above.
    
    Performance note: Even though the `ComparisonChain` caller always invokes its `compare` methods unconditionally, the `ComparisonChain` implementation stops calling its
    inputs' Comparable.compareTo compareTo and Comparator.compare compare methods as
    soon as one of them returns a nonzero result. This optimization is typically important only in
    the presence of expensive `compareTo` and `compare` implementations.
    
    See the Guava User Guide article on <a href=
    "https://github.com/google/guava/wiki/CommonObjectUtilitiesExplained#comparecompareto">`ComparisonChain`</a>.
    
    <h4 id="java8">Java 8+ equivalents</h4>
    
    If you are using Java version 8 or greater, you should generally use the static methods in Comparator instead of `ComparisonChain`. The example above can be implemented like this:
    
    ````import static java.util.Comparator.comparing;
    import static java.util.Comparator.nullsLast;
    import static java.util.Comparator.naturalOrder;
    
    ...
      private static final Comparator<Foo> COMPARATOR =
          comparing((Foo foo) -> foo.aString)
              .thenComparing(foo -> foo.anInt)
              .thenComparing(foo -> foo.anEnum, nullsLast(naturalOrder()));`
    
      `@Override``public int compareTo(Foo that) {
        return COMPARATOR.compare(this, that);`
    }```
    
    With method references it is more succinct: `comparing(Foo::aString)` for example.
    
    Using Comparator avoids certain types of bugs, for example when you meant to write
    `.compare(a.foo, b.foo)` but you actually wrote `.compare(a.foo, a.foo)` or `.compare(a.foo, b.bar)`. `ComparisonChain` also has a potential performance problem that
    `Comparator` doesn't: it evaluates all the parameters of all the `.compare` calls,
    even when the result of the comparison is already known from previous `.compare` calls.
    That can be expensive.

    Author(s)
    - Kevin Bourrillion

    Since
    - 2.0
    """

    @staticmethod
    def start() -> "ComparisonChain":
        """
        Begins a new chained comparison statement. See example in the class documentation.
        """
        ...


    def compare(self, left: "Comparable"[Any], right: "Comparable"[Any]) -> "ComparisonChain":
        """
        Compares two comparable objects as specified by Comparable.compareTo, *if* the
        result of this comparison chain has not already been determined.
        
        This method is declared to accept any 2 `Comparable` objects, even if they are not <a
        href="https://docs.oracle.com/javase/tutorial/collections/interfaces/order.html">mutually
        comparable</a>. If you pass objects that are not mutually comparable, this method may throw an
        exception. (The reason for this decision is lost to time, but the reason *might* be that
        we wanted to support legacy classes that implement the raw type `Comparable` (instead of
        implementing `Comparable<Foo>`) without producing warnings. If so, we would prefer today
        to produce warnings in that case, and we may change this method to do so in the future. Support
        for raw `Comparable` types in Guava in general is tracked as <a
        href="https://github.com/google/guava/issues/989">#989</a>.)

        Raises
        - ClassCastException: if the parameters are not mutually comparable
        """
        ...


    def compare(self, left: "T", right: "T", comparator: "Comparator"["T"]) -> "ComparisonChain":
        """
        Compares two objects using a comparator, *if* the result of this comparison chain has not
        already been determined.
        """
        ...


    def compare(self, left: int, right: int) -> "ComparisonChain":
        """
        Compares two `int` values as specified by Integer.compare, *if* the result
        of this comparison chain has not already been determined.
        """
        ...


    def compare(self, left: int, right: int) -> "ComparisonChain":
        """
        Compares two `long` values as specified by Long.compare, *if* the result of
        this comparison chain has not already been determined.
        """
        ...


    def compare(self, left: float, right: float) -> "ComparisonChain":
        """
        Compares two `float` values as specified by Float.compare, *if* the result
        of this comparison chain has not already been determined.
        """
        ...


    def compare(self, left: float, right: float) -> "ComparisonChain":
        """
        Compares two `double` values as specified by Double.compare, *if* the result
        of this comparison chain has not already been determined.
        """
        ...


    def compare(self, left: "Boolean", right: "Boolean") -> "ComparisonChain":
        """
        Discouraged synonym for .compareFalseFirst.

        Since
        - 19.0

        Deprecated
        - Use .compareFalseFirst; or, if the parameters passed are being either
            negated or reversed, undo the negation or reversal and use .compareTrueFirst.
        """
        ...


    def compareTrueFirst(self, left: bool, right: bool) -> "ComparisonChain":
        """
        Compares two `boolean` values, considering `True` to be less than `False`,
        *if* the result of this comparison chain has not already been determined.
        
        Java 8+ users: you can get the equivalent from Booleans.TrueFirst(). For example:
        
        ```
        Comparator.comparing(Foo::isBar, Booleans.TrueFirst())
        ```

        Since
        - 12.0
        """
        ...


    def compareFalseFirst(self, left: bool, right: bool) -> "ComparisonChain":
        """
        Compares two `boolean` values, considering `False` to be less than `True`,
        *if* the result of this comparison chain has not already been determined.
        
        Java 8+ users: you can get the equivalent from Booleans.FalseFirst(). For example:
        
        ```
        Comparator.comparing(Foo::isBar, Booleans.FalseFirst())
        ```

        Since
        - 12.0 (present as `compare` since 2.0)
        """
        ...


    def result(self) -> int:
        """
        Ends this comparison chain and returns its result: a value having the same sign as the first
        nonzero comparison result in the chain, or zero if every result was zero.
        """
        ...



"""
Python module generated from Java source file com.google.common.reflect.ClassPath

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import VisibleForTesting
from com.google.common.base import CharMatcher
from com.google.common.base import Splitter
from com.google.common.collect import FluentIterable
from com.google.common.collect import ImmutableList
from com.google.common.collect import ImmutableMap
from com.google.common.collect import ImmutableSet
from com.google.common.collect import Maps
from com.google.common.io import ByteSource
from com.google.common.io import CharSource
from com.google.common.io import Resources
from com.google.common.reflect import *
from java.io import File
from java.io import IOException
from java.net import MalformedURLException
from java.net import URISyntaxException
from java.net import URL
from java.net import URLClassLoader
from java.nio.charset import Charset
from java.util import Enumeration
from java.util import NoSuchElementException
from java.util.jar import Attributes
from java.util.jar import JarEntry
from java.util.jar import JarFile
from java.util.jar import Manifest
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class ClassPath:
    """
    Scans the source of a ClassLoader and finds all loadable classes and resources.
    
    <h2>Prefer <a href="https://github.com/classgraph/classgraph/wiki">ClassGraph</a> over `ClassPath`</h2>
    
    We recommend using <a href="https://github.com/classgraph/classgraph/wiki">ClassGraph</a>
    instead of `ClassPath`. ClassGraph improves upon `ClassPath` in several ways,
    including addressing many of its limitations. Limitations of `ClassPath` include:
    
    
      - It looks only for files and JARs in URLs available from URLClassLoader instances or
          the ClassLoader.getSystemClassLoader() system class loader. This means it does
          not look for classes in the *module path*.
      - It understands only `file:` URLs. This means that it does not understand <a
          href="https://openjdk.java.net/jeps/220">`jrt:/` URLs</a>, among <a
          href="https://github.com/classgraph/classgraph/wiki/Classpath-specification-mechanisms">others</a>.
      - It does not know how to look for classes when running under an Android VM. (ClassGraph does
          not support this directly, either, but ClassGraph documents how to <a
          href="https://github.com/classgraph/classgraph/wiki/Build-Time-Scanning">perform build-time
          classpath scanning and make the results available to an Android app</a>.)
      - Like all of Guava, it is not tested under Windows. We have gotten <a
          href="https://github.com/google/guava/issues/2130">a report of a specific bug under
          Windows</a>.
      - It <a href="https://github.com/google/guava/issues/2712">returns only one resource for a
          given path</a>, even if resources with that path appear in multiple jars or directories.
      - It assumes that <a href="https://github.com/google/guava/issues/3349">any class with a
          `$` in its name is a nested class</a>.
    
    
    <h2>`ClassPath` and symlinks</h2>
    
    In the case of directory classloaders, symlinks are supported but cycles are not traversed.
    This guarantees discovery of each *unique* loadable resource. However, not all possible
    aliases for resources on cyclic paths will be listed.

    Author(s)
    - Ben Yu

    Since
    - 14.0
    """

    @staticmethod
    def from(classloader: "ClassLoader") -> "ClassPath":
        """
        Returns a `ClassPath` representing all classes and resources loadable from `classloader` and its ancestor class loaders.
        
        **Warning:** `ClassPath` can find classes and resources only from:
        
        
          - URLClassLoader instances' `file:` URLs
          - the ClassLoader.getSystemClassLoader() system class loader. To search the
              system class loader even when it is not a URLClassLoader (as in Java 9), `ClassPath` searches the files from the `java.class.path` system property.

        Raises
        - IOException: if the attempt to read class path resources (jar files or directories)
            failed.
        """
        ...


    def getResources(self) -> "ImmutableSet"["ResourceInfo"]:
        """
        Returns all resources loadable from the current class path, including the class files of all
        loadable classes but excluding the "META-INF/MANIFEST.MF" file.
        """
        ...


    def getAllClasses(self) -> "ImmutableSet"["ClassInfo"]:
        """
        Returns all classes loadable from the current class path.

        Since
        - 16.0
        """
        ...


    def getTopLevelClasses(self) -> "ImmutableSet"["ClassInfo"]:
        """
        Returns all top level classes loadable from the current class path. Note that "top-level-ness"
        is determined heuristically by class name (see ClassInfo.isTopLevel).
        """
        ...


    def getTopLevelClasses(self, packageName: str) -> "ImmutableSet"["ClassInfo"]:
        """
        Returns all top level classes whose package name is `packageName`.
        """
        ...


    def getTopLevelClassesRecursive(self, packageName: str) -> "ImmutableSet"["ClassInfo"]:
        """
        Returns all top level classes whose package name is `packageName` or starts with `packageName` followed by a '.'.
        """
        ...


    class ResourceInfo:
        """
        Represents a class path resource that can be either a class file or any other resource file
        loadable from the class path.

        Since
        - 14.0
        """

        def url(self) -> "URL":
            """
            Returns the url identifying the resource.
            
            See ClassLoader.getResource

            Raises
            - NoSuchElementException: if the resource cannot be loaded through the class loader,
                despite physically existing in the class path.
            """
            ...


        def asByteSource(self) -> "ByteSource":
            """
            Returns a ByteSource view of the resource from which its bytes can be read.

            Raises
            - NoSuchElementException: if the resource cannot be loaded through the class loader,
                despite physically existing in the class path.

            Since
            - 20.0
            """
            ...


        def asCharSource(self, charset: "Charset") -> "CharSource":
            """
            Returns a CharSource view of the resource from which its bytes can be read as
            characters decoded with the given `charset`.

            Raises
            - NoSuchElementException: if the resource cannot be loaded through the class loader,
                despite physically existing in the class path.

            Since
            - 20.0
            """
            ...


        def getResourceName(self) -> str:
            """
            Returns the fully qualified name of the resource. Such as "com/mycomp/foo/bar.txt".
            """
            ...


        def hashCode(self) -> int:
            ...


        def equals(self, obj: "Object") -> bool:
            ...


        def toString(self) -> str:
            ...


    class ClassInfo(ResourceInfo):
        """
        Represents a class that can be loaded through .load.

        Since
        - 14.0
        """

        def getPackageName(self) -> str:
            """
            Returns the package name of the class, without attempting to load the class.
            
            Behaves similarly to `class.getPackage().`Package.getName() getName() but
            does not require the class (or package) to be loaded.
            
            But note that this method may behave differently for a class in the default package: For
            such classes, this method always returns an empty string. But under some version of Java,
            `class.getPackage().getName()` produces a `NullPointerException` because `class.getPackage()` returns `null`.
            """
            ...


        def getSimpleName(self) -> str:
            """
            Returns the simple name of the underlying class as given in the source code.
            
            Behaves similarly to Class.getSimpleName() but does not require the class to be
            loaded.
            
            But note that this class uses heuristics to identify the simple name. See a related
            discussion in <a href="https://github.com/google/guava/issues/3349">issue 3349</a>.
            """
            ...


        def getName(self) -> str:
            """
            Returns the fully qualified name of the class.
            
            Behaves identically to Class.getName() but does not require the class to be
            loaded.
            """
            ...


        def isTopLevel(self) -> bool:
            """
            Returns True if the class name "looks to be" top level (not nested), that is, it includes no
            '$' in the name. This method may return False for a top-level class that's intentionally
            named with the '$' character. If this is a concern, you could use .load and then
            check on the loaded Class object instead.

            Since
            - 30.1
            """
            ...


        def load(self) -> type[Any]:
            """
            Loads (but doesn't link or initialize) the class.

            Raises
            - LinkageError: when there were errors in loading classes that this class depends on.
                For example, NoClassDefFoundError.
            """
            ...


        def toString(self) -> str:
            ...



"""
Python module generated from Java source file com.google.common.graph.ValueGraphBuilder

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import Beta
from com.google.common.base import Optional
from com.google.common.graph import *
from com.google.errorprone.annotations import CanIgnoreReturnValue
from typing import Any, Callable, Iterable, Tuple


class ValueGraphBuilder(AbstractGraphBuilder):
    """
    A builder for constructing instances of MutableValueGraph or ImmutableValueGraph
    with user-defined properties.
    
    A `ValueGraph` built by this class has the following default properties:
    
    
      - does not allow self-loops
      - orders ValueGraph.nodes() in the order in which the elements were added (insertion
          order)
    
    
    `ValueGraph`s built by this class also guarantee that each collection-returning accessor
    returns a **(live) unmodifiable view**; see <a
    href="https://github.com/google/guava/wiki/GraphsExplained#accessor-behavior">the external
    documentation</a> for details.
    
    Examples of use:
    
    ````// Building a mutable value graph
    MutableValueGraph<String, Double> graph =
        ValueGraphBuilder.undirected().allowsSelfLoops(True).build();
    graph.putEdgeValue("San Francisco", "San Francisco", 0.0);
    graph.putEdgeValue("San Jose", "San Jose", 0.0);
    graph.putEdgeValue("San Francisco", "San Jose", 48.4);
    
    // Building an immutable value graph
    ImmutableValueGraph<String, Double> immutableGraph =
        ValueGraphBuilder.undirected()
            .allowsSelfLoops(True)
            .<String, Double>immutable()
            .putEdgeValue("San Francisco", "San Francisco", 0.0)
            .putEdgeValue("San Jose", "San Jose", 0.0)
            .putEdgeValue("San Francisco", "San Jose", 48.4)
            .build();````
    
    Type `<N>`: The most general node type this builder will support. This is normally `Object`
        unless it is constrained by using a method like .nodeOrder, or the builder is
        constructed based on an existing `ValueGraph` using .from(ValueGraph).
    
    Type `<V>`: The most general value type this builder will support. This is normally `Object`
        unless the builder is constructed based on an existing `Graph` using .from(ValueGraph).

    Author(s)
    - Joshua O'Madadhain

    Since
    - 20.0
    """

    @staticmethod
    def directed() -> "ValueGraphBuilder"["Object", "Object"]:
        """
        Returns a ValueGraphBuilder for building directed graphs.
        """
        ...


    @staticmethod
    def undirected() -> "ValueGraphBuilder"["Object", "Object"]:
        """
        Returns a ValueGraphBuilder for building undirected graphs.
        """
        ...


    @staticmethod
    def from(graph: "ValueGraph"["N", "V"]) -> "ValueGraphBuilder"["N", "V"]:
        """
        Returns a ValueGraphBuilder initialized with all properties queryable from `graph`.
        
        The "queryable" properties are those that are exposed through the ValueGraph
        interface, such as ValueGraph.isDirected(). Other properties, such as .expectedNodeCount(int), are not set in the new builder.
        """
        ...


    def immutable(self) -> "ImmutableValueGraph.Builder"["N1", "V1"]:
        """
        Returns an ImmutableValueGraph.Builder with the properties of this ValueGraphBuilder.
        
        The returned builder can be used for populating an ImmutableValueGraph.
        
        Note that the returned builder will always have .incidentEdgeOrder set to ElementOrder.stable(), regardless of the value that was set in this builder.

        Since
        - 28.0
        """
        ...


    def allowsSelfLoops(self, allowsSelfLoops: bool) -> "ValueGraphBuilder"["N", "V"]:
        """
        Specifies whether the graph will allow self-loops (edges that connect a node to itself).
        Attempting to add a self-loop to a graph that does not allow them will throw an UnsupportedOperationException.
        
        The default value is `False`.
        """
        ...


    def expectedNodeCount(self, expectedNodeCount: int) -> "ValueGraphBuilder"["N", "V"]:
        """
        Specifies the expected number of nodes in the graph.

        Raises
        - IllegalArgumentException: if `expectedNodeCount` is negative
        """
        ...


    def nodeOrder(self, nodeOrder: "ElementOrder"["N1"]) -> "ValueGraphBuilder"["N1", "V"]:
        """
        Specifies the order of iteration for the elements of Graph.nodes().
        
        The default value is ElementOrder.insertion() insertion order.
        """
        ...


    def incidentEdgeOrder(self, incidentEdgeOrder: "ElementOrder"["N1"]) -> "ValueGraphBuilder"["N1", "V"]:
        """
        Specifies the order of iteration for the elements of ValueGraph.edges(), ValueGraph.adjacentNodes(Object), ValueGraph.predecessors(Object), ValueGraph.successors(Object) and ValueGraph.incidentEdges(Object).
        
        The default value is ElementOrder.unordered() unordered for mutable graphs. For
        immutable graphs, this value is ignored; they always have a ElementOrder.stable()
        stable order.

        Raises
        - IllegalArgumentException: if `incidentEdgeOrder` is not either `ElementOrder.unordered()` or `ElementOrder.stable()`.

        Since
        - 29.0
        """
        ...


    def build(self) -> "MutableValueGraph"["N1", "V1"]:
        """
        Returns an empty MutableValueGraph with the properties of this ValueGraphBuilder.
        """
        ...



from .BiomeProvider import BiomeProvider
from .BiomeParameterPoint import BiomeParameterPoint
from .BlockPopulator import BlockPopulator
from .LimitedRegion import LimitedRegion
from .WorldInfo import WorldInfo
from .ChunkGenerator import ChunkGenerator


"""
Python module generated from Java source file org.joml.Math

Java source file obtained from artifact joml version 1.10.8

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.joml import *
from typing import Any, Callable, Iterable, Tuple


class Math:
    """
    Contains fast approximations of some java.lang.Math operations.
    
    By default, java.lang.Math methods will be used by all other JOML classes. In order to use the approximations in this class, start the JVM with the parameter `-Djoml.fastmath`.
    
    There are two algorithms for approximating sin/cos:
    <ol>
    - arithmetic <a href="http://www.java-gaming.org/topics/joml-1-8-0-release/37491/msg/361815/view.html#msg361815">polynomial approximation</a> contributed by roquendm 
    - theagentd's <a href="http://www.java-gaming.org/topics/extremely-fast-sine-cosine/36469/msg/346213/view.html#msg346213">linear interpolation</a> variant of Riven's algorithm from
    <a href="http://www.java-gaming.org/topics/extremely-fast-sine-cosine/36469/view.html">http://www.java-gaming.org/</a>
    </ol>
    By default, the first algorithm is being used. In order to use the second one, start the JVM with `-Djoml.sinLookup`. The lookup table bit length of the second algorithm can also be adjusted
    for improved accuracy via `-Djoml.sinLookup.bits=&lt;n&gt;`, where &lt;n&gt; is the number of bits of the lookup table.

    Author(s)
    - Kai Burjack
    """

    PI = java.lang.Math.PI
    PI_TIMES_2 = PI * 2.0
    PI_f = (float) java.lang.Math.PI
    PI_TIMES_2_f = PI_f * 2.0f
    PI_OVER_2 = PI * 0.5
    PI_OVER_2_f = (float) (PI * 0.5)
    PI_OVER_4 = PI * 0.25
    PI_OVER_4_f = (float) (PI * 0.25)
    ONE_OVER_PI = 1.0 / PI
    ONE_OVER_PI_f = (float) (1.0 / PI)


    @staticmethod
    def sin(rad: float) -> float:
        ...


    @staticmethod
    def sin(rad: float) -> float:
        ...


    @staticmethod
    def cos(rad: float) -> float:
        ...


    @staticmethod
    def cos(rad: float) -> float:
        ...


    @staticmethod
    def cosFromSin(sin: float, angle: float) -> float:
        ...


    @staticmethod
    def cosFromSin(sin: float, angle: float) -> float:
        ...


    @staticmethod
    def sqrt(r: float) -> float:
        ...


    @staticmethod
    def sqrt(r: float) -> float:
        ...


    @staticmethod
    def invsqrt(r: float) -> float:
        ...


    @staticmethod
    def invsqrt(r: float) -> float:
        ...


    @staticmethod
    def tan(r: float) -> float:
        ...


    @staticmethod
    def tan(r: float) -> float:
        ...


    @staticmethod
    def acos(r: float) -> float:
        ...


    @staticmethod
    def acos(r: float) -> float:
        ...


    @staticmethod
    def safeAcos(v: float) -> float:
        ...


    @staticmethod
    def safeAcos(v: float) -> float:
        ...


    @staticmethod
    def atan2(y: float, x: float) -> float:
        ...


    @staticmethod
    def atan2(y: float, x: float) -> float:
        ...


    @staticmethod
    def asin(r: float) -> float:
        ...


    @staticmethod
    def asin(r: float) -> float:
        ...


    @staticmethod
    def safeAsin(r: float) -> float:
        ...


    @staticmethod
    def safeAsin(r: float) -> float:
        ...


    @staticmethod
    def abs(r: float) -> float:
        ...


    @staticmethod
    def abs(r: float) -> float:
        ...


    @staticmethod
    def abs(r: int) -> int:
        ...


    @staticmethod
    def abs(r: int) -> int:
        ...


    @staticmethod
    def max(x: int, y: int) -> int:
        ...


    @staticmethod
    def min(x: int, y: int) -> int:
        ...


    @staticmethod
    def max(x: int, y: int) -> int:
        ...


    @staticmethod
    def min(x: int, y: int) -> int:
        ...


    @staticmethod
    def min(a: float, b: float) -> float:
        ...


    @staticmethod
    def min(a: float, b: float) -> float:
        ...


    @staticmethod
    def max(a: float, b: float) -> float:
        ...


    @staticmethod
    def max(a: float, b: float) -> float:
        ...


    @staticmethod
    def clamp(a: float, b: float, val: float) -> float:
        ...


    @staticmethod
    def clamp(a: float, b: float, val: float) -> float:
        ...


    @staticmethod
    def clamp(a: int, b: int, val: int) -> int:
        ...


    @staticmethod
    def clamp(a: int, b: int, val: int) -> int:
        ...


    @staticmethod
    def toRadians(angles: float) -> float:
        ...


    @staticmethod
    def toRadians(angles: float) -> float:
        ...


    @staticmethod
    def toDegrees(angles: float) -> float:
        ...


    @staticmethod
    def toDegrees(angles: float) -> float:
        ...


    @staticmethod
    def floor(v: float) -> float:
        ...


    @staticmethod
    def floor(v: float) -> float:
        ...


    @staticmethod
    def ceil(v: float) -> float:
        ...


    @staticmethod
    def ceil(v: float) -> float:
        ...


    @staticmethod
    def round(v: float) -> int:
        ...


    @staticmethod
    def round(v: float) -> int:
        ...


    @staticmethod
    def exp(a: float) -> float:
        ...


    @staticmethod
    def isFinite(d: float) -> bool:
        ...


    @staticmethod
    def isFinite(f: float) -> bool:
        ...


    @staticmethod
    def fma(a: float, b: float, c: float) -> float:
        ...


    @staticmethod
    def fma(a: float, b: float, c: float) -> float:
        ...


    @staticmethod
    def roundUsing(v: float, mode: int) -> int:
        ...


    @staticmethod
    def roundUsing(v: float, mode: int) -> int:
        ...


    @staticmethod
    def roundLongUsing(v: float, mode: int) -> int:
        ...


    @staticmethod
    def lerp(a: float, b: float, t: float) -> float:
        ...


    @staticmethod
    def lerp(a: float, b: float, t: float) -> float:
        ...


    @staticmethod
    def biLerp(q00: float, q10: float, q01: float, q11: float, tx: float, ty: float) -> float:
        ...


    @staticmethod
    def biLerp(q00: float, q10: float, q01: float, q11: float, tx: float, ty: float) -> float:
        ...


    @staticmethod
    def triLerp(q000: float, q100: float, q010: float, q110: float, q001: float, q101: float, q011: float, q111: float, tx: float, ty: float, tz: float) -> float:
        ...


    @staticmethod
    def triLerp(q000: float, q100: float, q010: float, q110: float, q001: float, q101: float, q011: float, q111: float, tx: float, ty: float, tz: float) -> float:
        ...


    @staticmethod
    def roundHalfEven(v: float) -> int:
        ...


    @staticmethod
    def roundHalfDown(v: float) -> int:
        ...


    @staticmethod
    def roundHalfUp(v: float) -> int:
        ...


    @staticmethod
    def roundHalfEven(v: float) -> int:
        ...


    @staticmethod
    def roundHalfDown(v: float) -> int:
        ...


    @staticmethod
    def roundHalfUp(v: float) -> int:
        ...


    @staticmethod
    def roundLongHalfEven(v: float) -> int:
        ...


    @staticmethod
    def roundLongHalfDown(v: float) -> int:
        ...


    @staticmethod
    def roundLongHalfUp(v: float) -> int:
        ...


    @staticmethod
    def random() -> float:
        ...


    @staticmethod
    def signum(v: float) -> float:
        ...


    @staticmethod
    def signum(v: float) -> float:
        ...


    @staticmethod
    def signum(v: int) -> int:
        ...


    @staticmethod
    def signum(v: int) -> int:
        ...



"""
Python module generated from Java source file org.bukkit.inventory.ItemRarity

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from org.bukkit.inventory import *
from typing import Any, Callable, Iterable, Tuple


class ItemRarity(Enum):
    """
    A item's rarity determines the default color of its name. This enum is
    ordered from least rare to most rare.
    """

    COMMON = 0
    """
    White item name.
    """
    UNCOMMON = 1
    """
    Yellow item name.
    """
    RARE = 2
    """
    Aqua item name.
    """
    EPIC = 3
    """
    Light purple item name.
    """



"""
Python module generated from Java source file com.google.gson.internal.bind.TypeAdapters

Java source file obtained from artifact gson version 2.11.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.gson import Gson
from com.google.gson import JsonArray
from com.google.gson import JsonElement
from com.google.gson import JsonIOException
from com.google.gson import JsonNull
from com.google.gson import JsonObject
from com.google.gson import JsonPrimitive
from com.google.gson import JsonSyntaxException
from com.google.gson import TypeAdapter
from com.google.gson import TypeAdapterFactory
from com.google.gson.annotations import SerializedName
from com.google.gson.internal import LazilyParsedNumber
from com.google.gson.internal import NumberLimits
from com.google.gson.internal import TroubleshootingGuide
from com.google.gson.internal.bind import *
from com.google.gson.reflect import TypeToken
from com.google.gson.stream import JsonReader
from com.google.gson.stream import JsonToken
from com.google.gson.stream import JsonWriter
from java.io import IOException
from java.lang.reflect import AccessibleObject
from java.lang.reflect import Field
from java.math import BigDecimal
from java.math import BigInteger
from java.net import InetAddress
from java.net import URI
from java.net import URISyntaxException
from java.net import URL
from java.security import AccessController
from java.security import PrivilegedAction
from java.util import ArrayDeque
from java.util import BitSet
from java.util import Calendar
from java.util import Currency
from java.util import Deque
from java.util import GregorianCalendar
from java.util import Locale
from java.util import StringTokenizer
from java.util import UUID
from java.util.concurrent.atomic import AtomicBoolean
from java.util.concurrent.atomic import AtomicInteger
from java.util.concurrent.atomic import AtomicIntegerArray
from typing import Any, Callable, Iterable, Tuple


class TypeAdapters:
    """
    Type adapters for basic types.
    """

    CLASS = TypeAdapter<Class>() {
    
        @Override
        public void write(JsonWriter out, Class value) throws IOException {
            throw UnsupportedOperationException("Attempted to serialize java.lang.Class: " + value.getName() + ". Forgot to register a type adapter?" + "\nSee " + TroubleshootingGuide.createUrl("java-lang-class-unsupported"));
        }
    
        @Override
        public Class read(JsonReader in) throws IOException {
            throw UnsupportedOperationException("Attempted to deserialize a java.lang.Class. Forgot to register a type adapter?" + "\nSee " + TroubleshootingGuide.createUrl("java-lang-class-unsupported"));
        }
    }.nullSafe()
    CLASS_FACTORY = newFactory(Class.class, CLASS)
    BIT_SET = TypeAdapter<BitSet>() {
    
        @Override
        public BitSet read(JsonReader in) throws IOException {
            BitSet bitset = BitSet();
            in.beginArray();
            int i = 0;
            JsonToken tokenType = in.peek();
            while (tokenType != JsonToken.END_ARRAY) {
                boolean set;
                switch(tokenType) {
                    case NUMBER:
                    case STRING:
                        int intValue = in.nextInt();
                        if (intValue == 0) {
                            set = false;
                        } else if (intValue == 1) {
                            set = true;
                        } else {
                            throw JsonSyntaxException("Invalid bitset value " + intValue + ", expected 0 or 1; at path " + in.getPreviousPath());
                        }
                        break;
                    case BOOLEAN:
                        set = in.nextBoolean();
                        break;
                    default:
                        throw JsonSyntaxException("Invalid bitset value type: " + tokenType + "; at path " + in.getPath());
                }
                if (set) {
                    bitset.set(i);
                }
                ++i;
                tokenType = in.peek();
            }
            in.endArray();
            return bitset;
        }
    
        @Override
        public void write(JsonWriter out, BitSet src) throws IOException {
            out.beginArray();
            for (int i = 0, length = src.length(); i < length; i++) {
                int value = src.get(i) ? 1 : 0;
                out.value(value);
            }
            out.endArray();
        }
    }.nullSafe()
    BIT_SET_FACTORY = newFactory(BitSet.class, BIT_SET)
    BOOLEAN = TypeAdapter<Boolean>() {
    
        @Override
        public Boolean read(JsonReader in) throws IOException {
            JsonToken peek = in.peek();
            if (peek == JsonToken.NULL) {
                in.nextNull();
                return null;
            } else if (peek == JsonToken.STRING) {
                // support strings for compatibility with GSON 1.7
                return Boolean.parseBoolean(in.nextString());
            }
            return in.nextBoolean();
        }
    
        @Override
        public void write(JsonWriter out, Boolean value) throws IOException {
            out.value(value);
        }
    }
    BOOLEAN_AS_STRING = TypeAdapter<Boolean>() {
    
        @Override
        public Boolean read(JsonReader in) throws IOException {
            if (in.peek() == JsonToken.NULL) {
                in.nextNull();
                return null;
            }
            return Boolean.valueOf(in.nextString());
        }
    
        @Override
        public void write(JsonWriter out, Boolean value) throws IOException {
            out.value(value == null ? "null" : value.toString());
        }
    }
    """
    Writes a boolean as a string. Useful for map keys, where booleans aren't otherwise permitted.
    """
    BOOLEAN_FACTORY = newFactory(boolean.class, Boolean.class, BOOLEAN)
    BYTE = TypeAdapter<Number>() {
    
        @Override
        public Number read(JsonReader in) throws IOException {
            if (in.peek() == JsonToken.NULL) {
                in.nextNull();
                return null;
            }
            int intValue;
            try {
                intValue = in.nextInt();
            } catch (NumberFormatException e) {
                throw JsonSyntaxException(e);
            }
            // Allow up to 255 to support unsigned values
            if (intValue > 255 || intValue < Byte.MIN_VALUE) {
                throw JsonSyntaxException("Lossy conversion from " + intValue + " to byte; at path " + in.getPreviousPath());
            }
            return (byte) intValue;
        }
    
        @Override
        public void write(JsonWriter out, Number value) throws IOException {
            if (value == null) {
                out.nullValue();
            } else {
                out.value(value.byteValue());
            }
        }
    }
    BYTE_FACTORY = newFactory(byte.class, Byte.class, BYTE)
    SHORT = TypeAdapter<Number>() {
    
        @Override
        public Number read(JsonReader in) throws IOException {
            if (in.peek() == JsonToken.NULL) {
                in.nextNull();
                return null;
            }
            int intValue;
            try {
                intValue = in.nextInt();
            } catch (NumberFormatException e) {
                throw JsonSyntaxException(e);
            }
            // Allow up to 65535 to support unsigned values
            if (intValue > 65535 || intValue < Short.MIN_VALUE) {
                throw JsonSyntaxException("Lossy conversion from " + intValue + " to short; at path " + in.getPreviousPath());
            }
            return (short) intValue;
        }
    
        @Override
        public void write(JsonWriter out, Number value) throws IOException {
            if (value == null) {
                out.nullValue();
            } else {
                out.value(value.shortValue());
            }
        }
    }
    SHORT_FACTORY = newFactory(short.class, Short.class, SHORT)
    INTEGER = TypeAdapter<Number>() {
    
        @Override
        public Number read(JsonReader in) throws IOException {
            if (in.peek() == JsonToken.NULL) {
                in.nextNull();
                return null;
            }
            try {
                return in.nextInt();
            } catch (NumberFormatException e) {
                throw JsonSyntaxException(e);
            }
        }
    
        @Override
        public void write(JsonWriter out, Number value) throws IOException {
            if (value == null) {
                out.nullValue();
            } else {
                out.value(value.intValue());
            }
        }
    }
    INTEGER_FACTORY = newFactory(int.class, Integer.class, INTEGER)
    ATOMIC_INTEGER = TypeAdapter<AtomicInteger>() {
    
        @Override
        public AtomicInteger read(JsonReader in) throws IOException {
            try {
                return AtomicInteger(in.nextInt());
            } catch (NumberFormatException e) {
                throw JsonSyntaxException(e);
            }
        }
    
        @Override
        public void write(JsonWriter out, AtomicInteger value) throws IOException {
            out.value(value.get());
        }
    }.nullSafe()
    ATOMIC_INTEGER_FACTORY = newFactory(AtomicInteger.class, TypeAdapters.ATOMIC_INTEGER)
    ATOMIC_BOOLEAN = TypeAdapter<AtomicBoolean>() {
    
        @Override
        public AtomicBoolean read(JsonReader in) throws IOException {
            return AtomicBoolean(in.nextBoolean());
        }
    
        @Override
        public void write(JsonWriter out, AtomicBoolean value) throws IOException {
            out.value(value.get());
        }
    }.nullSafe()
    ATOMIC_BOOLEAN_FACTORY = newFactory(AtomicBoolean.class, TypeAdapters.ATOMIC_BOOLEAN)
    ATOMIC_INTEGER_ARRAY = TypeAdapter<AtomicIntegerArray>() {
    
        @Override
        public AtomicIntegerArray read(JsonReader in) throws IOException {
            List<Integer> list = ArrayList<>();
            in.beginArray();
            while (in.hasNext()) {
                try {
                    int integer = in.nextInt();
                    list.add(integer);
                } catch (NumberFormatException e) {
                    throw JsonSyntaxException(e);
                }
            }
            in.endArray();
            int length = list.size();
            AtomicIntegerArray array = AtomicIntegerArray(length);
            for (int i = 0; i < length; ++i) {
                array.set(i, list.get(i));
            }
            return array;
        }
    
        @Override
        public void write(JsonWriter out, AtomicIntegerArray value) throws IOException {
            out.beginArray();
            for (int i = 0, length = value.length(); i < length; i++) {
                out.value(value.get(i));
            }
            out.endArray();
        }
    }.nullSafe()
    ATOMIC_INTEGER_ARRAY_FACTORY = newFactory(AtomicIntegerArray.class, TypeAdapters.ATOMIC_INTEGER_ARRAY)
    LONG = TypeAdapter<Number>() {
    
        @Override
        public Number read(JsonReader in) throws IOException {
            if (in.peek() == JsonToken.NULL) {
                in.nextNull();
                return null;
            }
            try {
                return in.nextLong();
            } catch (NumberFormatException e) {
                throw JsonSyntaxException(e);
            }
        }
    
        @Override
        public void write(JsonWriter out, Number value) throws IOException {
            if (value == null) {
                out.nullValue();
            } else {
                out.value(value.longValue());
            }
        }
    }
    FLOAT = TypeAdapter<Number>() {
    
        @Override
        public Number read(JsonReader in) throws IOException {
            if (in.peek() == JsonToken.NULL) {
                in.nextNull();
                return null;
            }
            return (float) in.nextDouble();
        }
    
        @Override
        public void write(JsonWriter out, Number value) throws IOException {
            if (value == null) {
                out.nullValue();
            } else {
                // For backward compatibility don't call `JsonWriter.value(float)` because that method
                // has been newly added and not all custom JsonWriter implementations might override
                // it yet
                Number floatNumber = value instanceof Float ? value : value.floatValue();
                out.value(floatNumber);
            }
        }
    }
    DOUBLE = TypeAdapter<Number>() {
    
        @Override
        public Number read(JsonReader in) throws IOException {
            if (in.peek() == JsonToken.NULL) {
                in.nextNull();
                return null;
            }
            return in.nextDouble();
        }
    
        @Override
        public void write(JsonWriter out, Number value) throws IOException {
            if (value == null) {
                out.nullValue();
            } else {
                out.value(value.doubleValue());
            }
        }
    }
    CHARACTER = TypeAdapter<Character>() {
    
        @Override
        public Character read(JsonReader in) throws IOException {
            if (in.peek() == JsonToken.NULL) {
                in.nextNull();
                return null;
            }
            String str = in.nextString();
            if (str.length() != 1) {
                throw JsonSyntaxException("Expecting character, got: " + str + "; at " + in.getPreviousPath());
            }
            return str.charAt(0);
        }
    
        @Override
        public void write(JsonWriter out, Character value) throws IOException {
            out.value(value == null ? null : String.valueOf(value));
        }
    }
    CHARACTER_FACTORY = newFactory(char.class, Character.class, CHARACTER)
    STRING = TypeAdapter<String>() {
    
        @Override
        public String read(JsonReader in) throws IOException {
            JsonToken peek = in.peek();
            if (peek == JsonToken.NULL) {
                in.nextNull();
                return null;
            }
            /* coerce booleans to strings for backwards compatibility */
            if (peek == JsonToken.BOOLEAN) {
                return Boolean.toString(in.nextBoolean());
            }
            return in.nextString();
        }
    
        @Override
        public void write(JsonWriter out, String value) throws IOException {
            out.value(value);
        }
    }
    BIG_DECIMAL = TypeAdapter<BigDecimal>() {
    
        @Override
        public BigDecimal read(JsonReader in) throws IOException {
            if (in.peek() == JsonToken.NULL) {
                in.nextNull();
                return null;
            }
            String s = in.nextString();
            try {
                return NumberLimits.parseBigDecimal(s);
            } catch (NumberFormatException e) {
                throw JsonSyntaxException("Failed parsing '" + s + "' as BigDecimal; at path " + in.getPreviousPath(), e);
            }
        }
    
        @Override
        public void write(JsonWriter out, BigDecimal value) throws IOException {
            out.value(value);
        }
    }
    BIG_INTEGER = TypeAdapter<BigInteger>() {
    
        @Override
        public BigInteger read(JsonReader in) throws IOException {
            if (in.peek() == JsonToken.NULL) {
                in.nextNull();
                return null;
            }
            String s = in.nextString();
            try {
                return NumberLimits.parseBigInteger(s);
            } catch (NumberFormatException e) {
                throw JsonSyntaxException("Failed parsing '" + s + "' as BigInteger; at path " + in.getPreviousPath(), e);
            }
        }
    
        @Override
        public void write(JsonWriter out, BigInteger value) throws IOException {
            out.value(value);
        }
    }
    LAZILY_PARSED_NUMBER = TypeAdapter<LazilyParsedNumber>() {
    
        // Normally users should not be able to access and deserialize LazilyParsedNumber because
        // it is an internal type, but implement this nonetheless in case there are legit corner
        // cases where this is possible
        @Override
        public LazilyParsedNumber read(JsonReader in) throws IOException {
            if (in.peek() == JsonToken.NULL) {
                in.nextNull();
                return null;
            }
            return LazilyParsedNumber(in.nextString());
        }
    
        @Override
        public void write(JsonWriter out, LazilyParsedNumber value) throws IOException {
            out.value(value);
        }
    }
    STRING_FACTORY = newFactory(String.class, STRING)
    STRING_BUILDER = TypeAdapter<StringBuilder>() {
    
        @Override
        public StringBuilder read(JsonReader in) throws IOException {
            if (in.peek() == JsonToken.NULL) {
                in.nextNull();
                return null;
            }
            return StringBuilder(in.nextString());
        }
    
        @Override
        public void write(JsonWriter out, StringBuilder value) throws IOException {
            out.value(value == null ? null : value.toString());
        }
    }
    STRING_BUILDER_FACTORY = newFactory(StringBuilder.class, STRING_BUILDER)
    STRING_BUFFER = TypeAdapter<StringBuffer>() {
    
        @Override
        public StringBuffer read(JsonReader in) throws IOException {
            if (in.peek() == JsonToken.NULL) {
                in.nextNull();
                return null;
            }
            return StringBuffer(in.nextString());
        }
    
        @Override
        public void write(JsonWriter out, StringBuffer value) throws IOException {
            out.value(value == null ? null : value.toString());
        }
    }
    STRING_BUFFER_FACTORY = newFactory(StringBuffer.class, STRING_BUFFER)
    URL = TypeAdapter<URL>() {
    
        @Override
        public URL read(JsonReader in) throws IOException {
            if (in.peek() == JsonToken.NULL) {
                in.nextNull();
                return null;
            }
            String nextString = in.nextString();
            return nextString.equals("null") ? null : URL(nextString);
        }
    
        @Override
        public void write(JsonWriter out, URL value) throws IOException {
            out.value(value == null ? null : value.toExternalForm());
        }
    }
    URL_FACTORY = newFactory(URL.class, URL)
    URI = TypeAdapter<URI>() {
    
        @Override
        public URI read(JsonReader in) throws IOException {
            if (in.peek() == JsonToken.NULL) {
                in.nextNull();
                return null;
            }
            try {
                String nextString = in.nextString();
                return nextString.equals("null") ? null : URI(nextString);
            } catch (URISyntaxException e) {
                throw JsonIOException(e);
            }
        }
    
        @Override
        public void write(JsonWriter out, URI value) throws IOException {
            out.value(value == null ? null : value.toASCIIString());
        }
    }
    URI_FACTORY = newFactory(URI.class, URI)
    INET_ADDRESS = TypeAdapter<InetAddress>() {
    
        @Override
        public InetAddress read(JsonReader in) throws IOException {
            if (in.peek() == JsonToken.NULL) {
                in.nextNull();
                return null;
            }
            // regrettably, this should have included both the host name and the host address
            // For compatibility, we use InetAddress.getByName rather than the possibly-better
            // .getAllByName
            @SuppressWarnings("AddressSelection")
            InetAddress addr = InetAddress.getByName(in.nextString());
            return addr;
        }
    
        @Override
        public void write(JsonWriter out, InetAddress value) throws IOException {
            out.value(value == null ? null : value.getHostAddress());
        }
    }
    INET_ADDRESS_FACTORY = newTypeHierarchyFactory(InetAddress.class, INET_ADDRESS)
    UUID = TypeAdapter<UUID>() {
    
        @Override
        public UUID read(JsonReader in) throws IOException {
            if (in.peek() == JsonToken.NULL) {
                in.nextNull();
                return null;
            }
            String s = in.nextString();
            try {
                return java.util.UUID.fromString(s);
            } catch (IllegalArgumentException e) {
                throw JsonSyntaxException("Failed parsing '" + s + "' as UUID; at path " + in.getPreviousPath(), e);
            }
        }
    
        @Override
        public void write(JsonWriter out, UUID value) throws IOException {
            out.value(value == null ? null : value.toString());
        }
    }
    UUID_FACTORY = newFactory(UUID.class, UUID)
    CURRENCY = TypeAdapter<Currency>() {
    
        @Override
        public Currency read(JsonReader in) throws IOException {
            String s = in.nextString();
            try {
                return Currency.getInstance(s);
            } catch (IllegalArgumentException e) {
                throw JsonSyntaxException("Failed parsing '" + s + "' as Currency; at path " + in.getPreviousPath(), e);
            }
        }
    
        @Override
        public void write(JsonWriter out, Currency value) throws IOException {
            out.value(value.getCurrencyCode());
        }
    }.nullSafe()
    CURRENCY_FACTORY = newFactory(Currency.class, CURRENCY)
    CALENDAR = TypeAdapter<Calendar>() {
    
        private static final String YEAR = "year";
    
        private static final String MONTH = "month";
    
        private static final String DAY_OF_MONTH = "dayOfMonth";
    
        private static final String HOUR_OF_DAY = "hourOfDay";
    
        private static final String MINUTE = "minute";
    
        private static final String SECOND = "second";
    
        @Override
        public Calendar read(JsonReader in) throws IOException {
            if (in.peek() == JsonToken.NULL) {
                in.nextNull();
                return null;
            }
            in.beginObject();
            int year = 0;
            int month = 0;
            int dayOfMonth = 0;
            int hourOfDay = 0;
            int minute = 0;
            int second = 0;
            while (in.peek() != JsonToken.END_OBJECT) {
                String name = in.nextName();
                int value = in.nextInt();
                switch(name) {
                    case YEAR:
                        year = value;
                        break;
                    case MONTH:
                        month = value;
                        break;
                    case DAY_OF_MONTH:
                        dayOfMonth = value;
                        break;
                    case HOUR_OF_DAY:
                        hourOfDay = value;
                        break;
                    case MINUTE:
                        minute = value;
                        break;
                    case SECOND:
                        second = value;
                        break;
                    default:
                }
            }
            in.endObject();
            return GregorianCalendar(year, month, dayOfMonth, hourOfDay, minute, second);
        }
    
        @Override
        public void write(JsonWriter out, Calendar value) throws IOException {
            if (value == null) {
                out.nullValue();
                return;
            }
            out.beginObject();
            out.name(YEAR);
            out.value(value.get(Calendar.YEAR));
            out.name(MONTH);
            out.value(value.get(Calendar.MONTH));
            out.name(DAY_OF_MONTH);
            out.value(value.get(Calendar.DAY_OF_MONTH));
            out.name(HOUR_OF_DAY);
            out.value(value.get(Calendar.HOUR_OF_DAY));
            out.name(MINUTE);
            out.value(value.get(Calendar.MINUTE));
            out.name(SECOND);
            out.value(value.get(Calendar.SECOND));
            out.endObject();
        }
    }
    CALENDAR_FACTORY = newFactoryForMultipleTypes(Calendar.class, GregorianCalendar.class, CALENDAR)
    LOCALE = TypeAdapter<Locale>() {
    
        @Override
        public Locale read(JsonReader in) throws IOException {
            if (in.peek() == JsonToken.NULL) {
                in.nextNull();
                return null;
            }
            String locale = in.nextString();
            StringTokenizer tokenizer = StringTokenizer(locale, "_");
            String language = null;
            String country = null;
            String variant = null;
            if (tokenizer.hasMoreElements()) {
                language = tokenizer.nextToken();
            }
            if (tokenizer.hasMoreElements()) {
                country = tokenizer.nextToken();
            }
            if (tokenizer.hasMoreElements()) {
                variant = tokenizer.nextToken();
            }
            if (country == null && variant == null) {
                return Locale(language);
            } else if (variant == null) {
                return Locale(language, country);
            } else {
                return Locale(language, country, variant);
            }
        }
    
        @Override
        public void write(JsonWriter out, Locale value) throws IOException {
            out.value(value == null ? null : value.toString());
        }
    }
    LOCALE_FACTORY = newFactory(Locale.class, LOCALE)
    JSON_ELEMENT = TypeAdapter<JsonElement>() {
    
        /**
         * Tries to begin reading a JSON array or JSON object, returning {@code null} if the next
         * element is neither of those.
         */
        private JsonElement tryBeginNesting(JsonReader in, JsonToken peeked) throws IOException {
            switch(peeked) {
                case BEGIN_ARRAY:
                    in.beginArray();
                    return JsonArray();
                case BEGIN_OBJECT:
                    in.beginObject();
                    return JsonObject();
                default:
                    return null;
            }
        }
    
        /**
         * Reads a {@link JsonElement} which cannot have any nested elements
         */
        private JsonElement readTerminal(JsonReader in, JsonToken peeked) throws IOException {
            switch(peeked) {
                case STRING:
                    return JsonPrimitive(in.nextString());
                case NUMBER:
                    String number = in.nextString();
                    return JsonPrimitive(LazilyParsedNumber(number));
                case BOOLEAN:
                    return JsonPrimitive(in.nextBoolean());
                case NULL:
                    in.nextNull();
                    return JsonNull.INSTANCE;
                default:
                    // When read(JsonReader) is called with JsonReader in invalid state
                    throw IllegalStateException("Unexpected token: " + peeked);
            }
        }
    
        @Override
        public JsonElement read(JsonReader in) throws IOException {
            if (in instanceof JsonTreeReader) {
                return ((JsonTreeReader) in).nextJsonElement();
            }
            // Either JsonArray or JsonObject
            JsonElement current;
            JsonToken peeked = in.peek();
            current = tryBeginNesting(in, peeked);
            if (current == null) {
                return readTerminal(in, peeked);
            }
            Deque<JsonElement> stack = ArrayDeque<>();
            while (true) {
                while (in.hasNext()) {
                    String name = null;
                    // Name is only used for JSON object members
                    if (current instanceof JsonObject) {
                        name = in.nextName();
                    }
                    peeked = in.peek();
                    JsonElement value = tryBeginNesting(in, peeked);
                    boolean isNesting = value != null;
                    if (value == null) {
                        value = readTerminal(in, peeked);
                    }
                    if (current instanceof JsonArray) {
                        ((JsonArray) current).add(value);
                    } else {
                        ((JsonObject) current).add(name, value);
                    }
                    if (isNesting) {
                        stack.addLast(current);
                        current = value;
                    }
                }
                // End current element
                if (current instanceof JsonArray) {
                    in.endArray();
                } else {
                    in.endObject();
                }
                if (stack.isEmpty()) {
                    return current;
                } else {
                    // Continue with enclosing element
                    current = stack.removeLast();
                }
            }
        }
    
        @Override
        public void write(JsonWriter out, JsonElement value) throws IOException {
            if (value == null || value.isJsonNull()) {
                out.nullValue();
            } else if (value.isJsonPrimitive()) {
                JsonPrimitive primitive = value.getAsJsonPrimitive();
                if (primitive.isNumber()) {
                    out.value(primitive.getAsNumber());
                } else if (primitive.isBoolean()) {
                    out.value(primitive.getAsBoolean());
                } else {
                    out.value(primitive.getAsString());
                }
            } else if (value.isJsonArray()) {
                out.beginArray();
                for (JsonElement e : value.getAsJsonArray()) {
                    write(out, e);
                }
                out.endArray();
            } else if (value.isJsonObject()) {
                out.beginObject();
                for (Map.Entry<String, JsonElement> e : value.getAsJsonObject().entrySet()) {
                    out.name(e.getKey());
                    write(out, e.getValue());
                }
                out.endObject();
            } else {
                throw IllegalArgumentException("Couldn't write " + value.getClass());
            }
        }
    }
    JSON_ELEMENT_FACTORY = newTypeHierarchyFactory(JsonElement.class, JSON_ELEMENT)
    ENUM_FACTORY = TypeAdapterFactory() {
    
        @Override
        public <T> TypeAdapter<T> create(Gson gson, TypeToken<T> typeToken) {
            Class<? super T> rawType = typeToken.getRawType();
            if (!Enum.class.isAssignableFrom(rawType) || rawType == Enum.class) {
                return null;
            }
            if (!rawType.isEnum()) {
                // handle anonymous subclasses
                rawType = rawType.getSuperclass();
            }
            @SuppressWarnings({ "rawtypes", "unchecked" })
            TypeAdapter<T> adapter = (TypeAdapter<T>) EnumTypeAdapter(rawType);
            return adapter;
        }
    }


    @staticmethod
    def newFactory(type: "TypeToken"["TT"], typeAdapter: "TypeAdapter"["TT"]) -> "TypeAdapterFactory":
        ...


    @staticmethod
    def newFactory(type: type["TT"], typeAdapter: "TypeAdapter"["TT"]) -> "TypeAdapterFactory":
        ...


    @staticmethod
    def newFactory(unboxed: type["TT"], boxed: type["TT"], typeAdapter: "TypeAdapter"["TT"]) -> "TypeAdapterFactory":
        ...


    @staticmethod
    def newFactoryForMultipleTypes(base: type["TT"], sub: type["TT"], typeAdapter: "TypeAdapter"["TT"]) -> "TypeAdapterFactory":
        ...


    @staticmethod
    def newTypeHierarchyFactory(clazz: type["T1"], typeAdapter: "TypeAdapter"["T1"]) -> "TypeAdapterFactory":
        """
        Returns a factory for all subtypes of `typeAdapter`. We do a runtime check to confirm
        that the deserialized type matches the type requested.
        """
        ...



"""
Python module generated from Java source file org.bukkit.block.structure.Mirror

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from org.bukkit.block.structure import *
from typing import Any, Callable, Iterable, Tuple


class Mirror(Enum):
    """
    Represents how a org.bukkit.block.Structure can be mirrored upon
    being loaded.
    """

    NONE = 0
    """
    No mirroring.
    
    Positive X to Positive Z
    """
    LEFT_RIGHT = 1
    """
    Structure is mirrored left to right.
    
    Similar to looking in a mirror. Positive X to Negative Z
    """
    FRONT_BACK = 2
    """
    Structure is mirrored front to back.
    
    Positive Z to Negative X
    """
